//////////////////////////////////////////////////////////////////////
////                                                              ////
////  Test bench                                                  ////
////                                                              ////
////  This file is part of the OpenRISC 1200 project              ////
////  http://www.opencores.org/cores/or1k/                        ////
////                                                              ////
////  Description                                                 ////
////  Test bench using on OR1200 and wb_sram.                     ////
////                                                              ////
////  To Do:                                                      ////
////   None.                                                      ////
////                                                              ////
////  Author(s):                                                  ////
////      - Damjan Lampret, lampret@opencores.org                 ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// $Log$
//

`include "general.h"

module tb_or1200;

`include "monitor.h"

//
// System connections
//
reg     clk;
reg     rst;

//
// Interconnect between OR1200 and iwb_ram
//
wire            iwb_ack;
wire            iwb_cyc;
wire    [15:0]  iwb_adr;
wire    [31:0]  iwb_dat_risc;
wire    [31:0]  iwb_dat_mem;
wire            iwb_err;
wire            iwb_rty;
wire    [3:0]   iwb_sel;
wire            iwb_stb;
wire            iwb_we;

//
// Interconnect between OR1200 and dwb_ram
//
wire            dwb_ack;
wire            dwb_cyc;
wire    [15:0]  dwb_adr;
wire    [31:0]  dwb_dat_risc;
wire    [31:0]  dwb_dat_mem;
wire            dwb_err;
wire            dwb_rty;
wire    [3:0]   dwb_sel;
wire            dwb_stb;
wire            dwb_we;

integer i;

//
// Initial state
//
initial
begin
        clk = 1'b0;
end

//
// Reset interval
//
initial
begin
	for(i = 0; i < 32; i = i + 1) begin
		risc.cpu.rf.rf_a.mem[i] = 0;
		risc.cpu.rf.rf_b.mem[i] = 0;
	end
	for(i = 0; i < 512; i = i + 1) begin
		risc.ic.ic_tag.ic_tag0.mem[i] = 19'h1234;
	end
	for(i = 0; i < 2048; i = i + 1) begin
		risc.ic.ic_ram.ic_ram0.mem[i] = 32'b0;
	end
	for(i = 0; i < 512; i = i + 1) begin
		risc.dc.dc_tag.dc_tag0.mem[i] = 19'h1234;
	end
	for(i = 0; i < 2048; i = i + 1) begin
		risc.dc.dc_ram.dc_ram0.mem[i] = 8'b0;
		risc.dc.dc_ram.dc_ram1.mem[i] = 8'b0;
		risc.dc.dc_ram.dc_ram2.mem[i] = 8'b0;
		risc.dc.dc_ram.dc_ram3.mem[i] = 8'b0;
	end
	risc.ic.ic_tag.ic_tag0.LAST_qi = 19'h1234;
	risc.dc.dc_tag.dc_tag0.LAST_qi = 19'h1234;
	risc.ic.ic_tag.ic_tag0.d_reg = 19'h1234;
	risc.dc.dc_tag.dc_tag0.d_reg = 19'h1234;
	risc.ic.ic_tag.ic_tag0.qi = 19'h1234;
	risc.dc.dc_tag.dc_tag0.qi = 19'h1234;
	$dumpfile("dump.vcd");
	$dumpvars(20,tb_or1200);
        rst = 1'b0;
        #1 rst = 1'b1;
        #25 rst = 1'b0;
//        #1000 $finish;
end

//
// Clock generation
//
always
        #10 clk = ~clk;


or1200 risc(
        .clk(clk),
        .rst(rst),
        .irq(0),
        .clkdiv_by_2(1'b0),

	// Instruction WISHBONE interface
	.iwb_clk_i(clk),
	.iwb_rst_i(rst),
	.iwb_ack_i(iwb_ack),
	.iwb_err_i(iwb_err),
	.iwb_rty_i(iwb_rty),
	.iwb_dat_i(iwb_dat_risc),
	.iwb_cyc_o(iwb_cyc),
	.iwb_adr_o(iwb_adr),
	.iwb_stb_o(iwb_stb),
	.iwb_we_o(iwb_we),
	.iwb_sel_o(iwb_sel),
	.iwb_dat_o(iwb_dat_mem),

	// Data WISHBONE interface
	.dwb_clk_i(clk),
	.dwb_rst_i(rst),
	.dwb_ack_i(dwb_ack),
	.dwb_err_i(dwb_err),
	.dwb_rty_i(dwb_rty),
	.dwb_dat_i(dwb_dat_risc),
	.dwb_cyc_o(dwb_cyc),
	.dwb_adr_o(dwb_adr),
	.dwb_stb_o(dwb_stb),
	.dwb_we_o(dwb_we),
	.dwb_sel_o(dwb_sel),
	.dwb_dat_o(dwb_dat_mem),

        // These connect RISC's Trace port to external Trace port pads
        .tp_dir_in(1'b0),
        .tp_sel(2'b0),
        .tp_in(32'b0),
        .tp_out(),

        // These connect RISC's module interface to peripherals
        .mod_addr(),
        .mod_from_risc(),
        .mod_to_risc(32'b0)
);

//
// Instatiation of Slave WISHBONE Dual-port Synchronous SRAM
//
wb_sram iwb_ram(
	// WISHBONE interface 1
	.wb1_clk_i(clk),
	.wb1_rst_i(rst),

	.wb1_cyc_i(iwb_cyc),
	.wb1_adr_i(iwb_adr),
	.wb1_dat_i(iwb_dat_mem),
	.wb1_stb_i(iwb_stb),
	.wb1_we_i(iwb_we),
	.wb1_sel_i(iwb_sel),

	.wb1_ack_o(iwb_ack),
	.wb1_err_o(iwb_err),
	.wb1_rty_o(iwb_rty),
	.wb1_dat_o(iwb_dat_risc),

	// WISHBONE interface 2
	.wb2_clk_i(clk),
	.wb2_rst_i(rst),

	.wb2_cyc_i(dwb_cyc),
	.wb2_adr_i(dwb_adr),
	.wb2_dat_i(dwb_dat_mem),
	.wb2_stb_i(dwb_stb),
	.wb2_we_i(dwb_we),
	.wb2_sel_i(dwb_sel),

	.wb2_ack_o(dwb_ack),
	.wb2_err_o(dwb_err),
	.wb2_rty_o(dwb_rty),
	.wb2_dat_o(dwb_dat_risc)
);

endmodule
