//////////////////////////////////////////////////////////////////////
////                                                              ////
////  OR1200's DC RAMs                                            ////
////                                                              ////
////  This file is part of the OpenRISC 1200 project              ////
////  http://www.opencores.org/cores/or1k/                        ////
////                                                              ////
////  Description                                                 ////
////  Instatiation of DC RAM blocks.                              ////
////                                                              ////
////  To Do:                                                      ////
////   - make it smaller and faster                               ////
////                                                              ////
////  Author(s):                                                  ////
////      - Damjan Lampret, lampret@opencores.org                 ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// $Log$
//

`include "general.h"

module dc_ram(clk, addr, we, datain, dataout, tp2w, tpdw);

parameter dw = `OPERAND_WIDTH;
parameter aw = 11;

input [dw-1:0] datain;
output [dw-1:0] dataout;

input [aw-1:0] addr;
input [3:0] we;

input [`TP2W_WIDTH-1:0] tp2w;
input [31:0] tpdw;

input clk;

wire  high, low;

wire [dw-1:0] t_data;

assign high = 1;
assign low = 0;

assign t_data = 32'b0;

art_hssp_2048x8 dc_ram0(
   .q(dataout[7:0]),

   .censq(),
   .oensq(),
   .wensq(),
   .asq(),
   .tq(),

   .clk(clk),
   .cen(low),
   .oen(low),
   .wen(~we[0]),
   .a(addr),
   .d(datain[7:0]),
   .tis(tp2w[`TP2W_DCR_TIS]),
   .tms(tp2w[`TP2W_DCR_TMS]),
   .tcen(tp2w[`TP2W_DCR_TCEN]),
   .toen(tp2w[`TP2W_DCR_TOEN]),
   .tqoen(tp2w[`TP2W_DCR_TQOEN]),
   .twen(tp2w[`TP2W_DCR_TWEN]),
   .ta(tp2w[`TP2W_DCR_TA]),
   .td(tpdw[7:0])
);

art_hssp_2048x8 dc_ram1(
   .q(dataout[15:8]),

   .censq(),
   .oensq(),
   .wensq(),
   .asq(),
   .tq(),

   .clk(clk),
   .cen(low),
   .oen(low),
   .wen(~we[1]),
   .a(addr),
   .d(datain[15:8]),
   .tis(tp2w[`TP2W_DCR_TIS]),
   .tms(tp2w[`TP2W_DCR_TMS]),
   .tcen(tp2w[`TP2W_DCR_TCEN]),
   .toen(tp2w[`TP2W_DCR_TOEN]),
   .tqoen(tp2w[`TP2W_DCR_TQOEN]),
   .twen(tp2w[`TP2W_DCR_TWEN]),
   .ta(tp2w[`TP2W_DCR_TA]),
   .td(tpdw[15:8])
);

art_hssp_2048x8 dc_ram2(
   .q(dataout[23:16]),

   .censq(),
   .oensq(),
   .wensq(),
   .asq(),
   .tq(),

   .clk(clk),
   .cen(low),
   .oen(low),
   .wen(~we[2]),
   .a(addr),
   .d(datain[23:16]),
   .tis(tp2w[`TP2W_DCR_TIS]),
   .tms(tp2w[`TP2W_DCR_TMS]),
   .tcen(tp2w[`TP2W_DCR_TCEN]),
   .toen(tp2w[`TP2W_DCR_TOEN]),
   .tqoen(tp2w[`TP2W_DCR_TQOEN]),
   .twen(tp2w[`TP2W_DCR_TWEN]),
   .ta(tp2w[`TP2W_DCR_TA]),
   .td(tpdw[23:16])
);

art_hssp_2048x8 dc_ram3(
   .q(dataout[31:24]),

   .censq(),
   .oensq(),
   .wensq(),
   .asq(),
   .tq(),

   .clk(clk),
   .cen(low),
   .oen(low),
   .wen(~we[3]),
   .a(addr),
   .d(datain[31:24]),
   .tis(tp2w[`TP2W_DCR_TIS]),
   .tms(tp2w[`TP2W_DCR_TMS]),
   .tcen(tp2w[`TP2W_DCR_TCEN]),
   .toen(tp2w[`TP2W_DCR_TOEN]),
   .tqoen(tp2w[`TP2W_DCR_TQOEN]),
   .twen(tp2w[`TP2W_DCR_TWEN]),
   .ta(tp2w[`TP2W_DCR_TA]),
   .td(tpdw[31:24])
);

endmodule
