//////////////////////////////////////////////////////////////////////
////                                                              ////
////  OR1200's DTLB                                               ////
////                                                              ////
////  This file is part of the OpenRISC 1200 project              ////
////  http://www.opencores.org/cores/or1k/                        ////
////                                                              ////
////  Description                                                 ////
////  Data TLB                                                    ////
////                                                              ////
////  To Do:                                                      ////
////   - make it smaller and faster                               ////
////                                                              ////
////  Author(s):                                                  ////
////      - Damjan Lampret, lampret@opencores.org                 ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// $Log$
//

`include "general.h"

// DTLB unit
module dtlb(clk, rst, we, tlb_en, datain, vaddr, paddr, cache_en, tlb_miss, tp3w, tpdw);

input clk;
input rst;
input we;
input tlb_en;

input [31:0] datain;
input [31:0] vaddr;
output [31:0] paddr;
output cache_en;
output tlb_miss;

input [`TP3W_WIDTH-1:0] tp3w;
input [31:0] tpdw;

wire [33:0] to_ram;
wire [33:0] from_ram;

reg [31:0] saved_vaddr;

// datain[31:12] = PPN
// datain[0] = cache enable
assign to_ram = { datain[31:12], datain[0], vaddr[31:19] };

// If TLB disabled then PADDR = VADDR
assign paddr = (tlb_en == 1'b1) ? { from_ram[33:14], saved_vaddr[11:0] } : saved_vaddr;

// Per page cache control
assign cache_en = from_ram[13];

// Comparison of stored TLB tag with current VADDR's page index
// Lower 7 bits of VADDR's page index are ignored since we index
// TLB entries with them so they always match
assign tlb_miss = (from_ram[12:0] != saved_vaddr[31:19]) ? 1'b1 : 1'b0;

// Save vaddr
always @(posedge clk or posedge rst)
	if (rst)
		saved_vaddr <= #1 32'b0;
	else
		saved_vaddr <= #1 vaddr;

art_hssp_128x34 dtlb0(
   .q(from_ram),

   .censq(),
   .oensq(),
   .wensq(),
   .asq(),
   .tq(),

   .clk(clk),
   .cen(1'b0),
   .oen(1'b0),
   .wen(~we),
   .a(vaddr[18:12]),
   .d(to_ram),
   .tis(tp3w[`TP4W_IMT_TIS]),
   .tms(tp3w[`TP4W_IMT_TMS]),
   .tcen(tp3w[`TP4W_IMT_TCEN]),
   .toen(tp3w[`TP4W_IMT_TOEN]),
   .tqoen(tp3w[`TP4W_IMT_TQOEN]),
   .twen(tp3w[`TP4W_IMT_TWEN]),
   .ta(tp3w[`TP4W_IMT_TA]),
   .td()
);

endmodule
