//////////////////////////////////////////////////////////////////////
////                                                              ////
////  OR1200's Old interrupt controller (pic.v will reaplce it)   ////
////                                                              ////
////  This file is part of the OpenRISC 1200 project              ////
////  http://www.opencores.org/cores/or1k/                        ////
////                                                              ////
////  Description                                                 ////
////  Old interrupt controller                                    ////
////                                                              ////
////  To Do:                                                      ////
////   - make it smaller and faster                               ////
////                                                              ////
////  Author(s):                                                  ////
////      - Damjan Lampret, lampret@opencores.org                 ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
////                                                              ////
//// Copyright (C) 2000 Authors and OPENCORES.ORG                 ////
////                                                              ////
//// This source file may be used and distributed without         ////
//// restriction provided that this copyright statement is not    ////
//// removed from the file and that any derivative work contains  ////
//// the original copyright notice and the associated disclaimer. ////
////                                                              ////
//// This source file is free software; you can redistribute it   ////
//// and/or modify it under the terms of the GNU Lesser General   ////
//// Public License as published by the Free Software Foundation; ////
//// either version 2.1 of the License, or (at your option) any   ////
//// later version.                                               ////
////                                                              ////
//// This source is distributed in the hope that it will be       ////
//// useful, but WITHOUT ANY WARRANTY; without even the implied   ////
//// warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR      ////
//// PURPOSE.  See the GNU Lesser General Public License for more ////
//// details.                                                     ////
////                                                              ////
//// You should have received a copy of the GNU Lesser General    ////
//// Public License along with this source; if not, download it   ////
//// from http://www.opencores.org/lgpl.shtml                     ////
////                                                              ////
//////////////////////////////////////////////////////////////////////
//
// CVS Revision History
//
// $Log$
//

`include "general.h"

// Interrupt controller
//
// Prioritizes interrupt inputs and generates int_low and int_high interrupt exceptions

module intc(clk, rst, irq, int_low, int_high,
	cs, addr, datain
);

input clk;
input rst;

// Interrupt request inputs
input [`IRQ_WIDTH-1:0] irq;

// Exceptions signaled to exception unit
output int_low;
output int_high;

// INTC management
input cs;
input [1:0] addr;
input [31:0] datain;

reg int_low, int_high;

// This registers selects which irq inputs generate int_high exception
reg [`IRQ_WIDTH-1:0] hiselect;

// This registers masks irq inputs (if bit is '0' then corresponding irq is masked)
reg [`IRQ_WIDTH-1:0] irqen;

always @(posedge clk or posedge rst)
	if (rst) begin
		irqen <= #1 `IRQ_WIDTH'b0;
		hiselect <= #1 `IRQ_WIDTH'b0;
	end
	else if (cs)
		casex (addr)
			2'b00:
				irqen <= #1 datain;
			2'bx1:
				hiselect <= #1 datain;
		endcase

always @(posedge clk or posedge rst)
	if (rst) begin
		int_low <= #1 1'b0;
		int_high <= #1 1'b0;
	end
	else begin
		if (hiselect & irqen & irq)
			int_high <= #1 1'b1;
//			#1003 int_high <= 1'b1;
//			#20 int_high <= 1'b0;
		if (~hiselect & irqen & irq)
			int_low <= #1 1'b1;
	end

endmodule
