//
//      CONFIDENTIAL AND PROPRIETARY SOFTWARE/DATA OF ARTISAN COMPONENTS, INC.
//      
//      Copyright (c) 2000 Artisan Components, Inc.  All Rights Reserved.
//      
//      Use of this Software/Data is subject to the terms and conditions of
//      the applicable license agreement between Artisan Components, Inc. and
//      Taiwan Semiconductor Manufacturing Ltd..  In addition, this Software/Data
//      is protected by copyright law and international treaties.
//      
//      The copyright notice(s) in this Software/Data does not indicate actual
//      or intended publication of this Software/Data.
//
//      Verilog model for 
//
//      Instance Name:  art_hdsp_2048x32
//      Words:          2048
//      Word Width:     32
//      Pipeline:       No
//
//      Creation Date:  2000-12-28 11:02:28Z
//      Version: 	2000Q3V0
//
//      Verified With: Cadence Verilog-XL
//
//      Modeling Assumptions: This model supports full gate level simulation
//          including proper x-handling and timing check behavior.  Unit
//          delay timing is included in the model. Back-annotation of SDF
//          (v2.1) is supported.  SDF can be created utilyzing the delay
//          calculation views provided with this generator and supported
//          delay calculators.  All buses are modeled [MSB:LSB].  All 
//          ports are padded with Verilog primitives.
//
//      Modeling Limitations: The output hold function has been deleted
//          completely from this model.  Most Verilog simulators are 
//          incapable of scheduling more than 1 event on the rising 
//          edge of clock.  Therefore, it is impossible to model
//          the output hold (to x) action correctly.  It is necessary
//          to run static path timing tools using Artisan supplied
//          timing models to insure that the output hold time is
//          sufficient enough to not violate hold time constraints
//          of downstream flip-flops.
//
//      Known Bugs: None.
//
//      Known Work Arounds: N/A
//
`timescale 1 ns/10 ps
`celldefine
module art_hdsp_2048x32 (
   q,
   clk,
   cen,
   wen,
   a,
   d
);
   parameter		   BITS = 32;
   parameter		   word_depth = 2048;
   parameter		   addr_width = 11;
   parameter		   wordx = {BITS{1'bx}};
   parameter		   addrx = {addr_width{1'bx}};
	
   output [31:0] q;
   input clk;
   input cen;
   input wen;
   input [10:0] a;
   input [31:0] d;

   reg [BITS-1:0]	   mem [word_depth-1:0];

   reg			   NOT_cen;
   reg			   NOT_wen;

   reg			   NOT_a0;
   reg			   NOT_a1;
   reg			   NOT_a2;
   reg			   NOT_a3;
   reg			   NOT_a4;
   reg			   NOT_a5;
   reg			   NOT_a6;
   reg			   NOT_a7;
   reg			   NOT_a8;
   reg			   NOT_a9;
   reg			   NOT_a10;
   reg [addr_width-1:0]	   NOT_a;
   reg			   NOT_d0;
   reg			   NOT_d1;
   reg			   NOT_d2;
   reg			   NOT_d3;
   reg			   NOT_d4;
   reg			   NOT_d5;
   reg			   NOT_d6;
   reg			   NOT_d7;
   reg			   NOT_d8;
   reg			   NOT_d9;
   reg			   NOT_d10;
   reg			   NOT_d11;
   reg			   NOT_d12;
   reg			   NOT_d13;
   reg			   NOT_d14;
   reg			   NOT_d15;
   reg			   NOT_d16;
   reg			   NOT_d17;
   reg			   NOT_d18;
   reg			   NOT_d19;
   reg			   NOT_d20;
   reg			   NOT_d21;
   reg			   NOT_d22;
   reg			   NOT_d23;
   reg			   NOT_d24;
   reg			   NOT_d25;
   reg			   NOT_d26;
   reg			   NOT_d27;
   reg			   NOT_d28;
   reg			   NOT_d29;
   reg			   NOT_d30;
   reg			   NOT_d31;
   reg [BITS-1:0]	   NOT_d;
   reg			   NOT_clk_PER;
   reg			   NOT_clk_MINH;
   reg			   NOT_clk_MINL;

   reg			   LAST_NOT_cen;
   reg			   LAST_NOT_wen;
   reg [addr_width-1:0]	   LAST_NOT_a;
   reg [BITS-1:0]	   LAST_NOT_d;
   reg			   LAST_NOT_clk_PER;
   reg			   LAST_NOT_clk_MINH;
   reg			   LAST_NOT_clk_MINL;


   wire [BITS-1:0]   _q;
   wire [addr_width-1:0]   _a;
   wire			   _clk;
   wire			   _cen;
   wire                    _wen;

   wire [BITS-1:0]   _d;
   wire                    re_flag;
   wire                    re_data_flag;


   reg			   LATCHED_cen;
   reg	                  LATCHED_wen;
   reg [addr_width-1:0]	   LATCHED_a;
   reg [BITS-1:0]	   LATCHED_d;

   reg			   ceni;
   reg           	   weni;
   reg [addr_width-1:0]	   ai;
   reg [BITS-1:0]	   di;
   reg [BITS-1:0]	   qi;
   reg [BITS-1:0]	   LAST_qi;



   reg			   LAST_clk;





   task update_notifier_buses;
   begin
      NOT_a = {
               NOT_a10,
               NOT_a9,
               NOT_a8,
               NOT_a7,
               NOT_a6,
               NOT_a5,
               NOT_a4,
               NOT_a3,
               NOT_a2,
               NOT_a1,
               NOT_a0};
      NOT_d = {
               NOT_d31,
               NOT_d30,
               NOT_d29,
               NOT_d28,
               NOT_d27,
               NOT_d26,
               NOT_d25,
               NOT_d24,
               NOT_d23,
               NOT_d22,
               NOT_d21,
               NOT_d20,
               NOT_d19,
               NOT_d18,
               NOT_d17,
               NOT_d16,
               NOT_d15,
               NOT_d14,
               NOT_d13,
               NOT_d12,
               NOT_d11,
               NOT_d10,
               NOT_d9,
               NOT_d8,
               NOT_d7,
               NOT_d6,
               NOT_d5,
               NOT_d4,
               NOT_d3,
               NOT_d2,
               NOT_d1,
               NOT_d0};
   end
   endtask

   task mem_cycle;
   begin
      casez({weni,ceni})

	2'b10: begin
	   read_mem(1,0);
	end
	2'b00: begin
	   write_mem(ai,di);
	   read_mem(0,0);
	end
	2'b?1: ;
	2'b1x: begin
	   read_mem(0,1);
	end
	2'bx0: begin
	   write_mem_x(ai);
	   read_mem(0,1);
	end
	2'b0x,
	2'bxx: begin
	   write_mem_x(ai);
	   read_mem(0,1);
	end
      endcase
   end
   endtask
      

   task update_last_notifiers;
   begin
      LAST_NOT_a = NOT_a;
      LAST_NOT_d = NOT_d;
      LAST_NOT_wen = NOT_wen;
      LAST_NOT_cen = NOT_cen;
      LAST_NOT_clk_PER = NOT_clk_PER;
      LAST_NOT_clk_MINH = NOT_clk_MINH;
      LAST_NOT_clk_MINL = NOT_clk_MINL;
   end
   endtask

   task latch_inputs;
   begin
      LATCHED_a = _a ;
      LATCHED_d = _d ;
      LATCHED_wen = _wen ;
      LATCHED_cen = _cen ;
      LAST_qi = qi;
   end
   endtask


   task update_logic;
   begin
      ceni = LATCHED_cen;
      weni = LATCHED_wen;
      ai = LATCHED_a;
      di = LATCHED_d;
   end
   endtask



   task x_inputs;
      integer n;
   begin
      for (n=0; n<addr_width; n=n+1)
	 begin
	    LATCHED_a[n] = (NOT_a[n]!==LAST_NOT_a[n]) ? 1'bx : LATCHED_a[n] ;
	 end
      for (n=0; n<BITS; n=n+1)
	 begin
	    LATCHED_d[n] = (NOT_d[n]!==LAST_NOT_d[n]) ? 1'bx : LATCHED_d[n] ;
	 end
      LATCHED_wen = (NOT_wen!==LAST_NOT_wen) ? 1'bx : LATCHED_wen ;

      LATCHED_cen = (NOT_cen!==LAST_NOT_cen) ? 1'bx : LATCHED_cen ;
   end
   endtask

   task read_mem;
      input r_wb;
      input xflag;
   begin
      if (r_wb)
	 begin
	    if (valid_address(ai))
	       begin
                     qi=mem[ai];
	       end
	    else
	       begin
		  qi=wordx;
	       end
	 end
      else
	 begin
	    if (xflag)
	       begin
		  qi=wordx;
	       end
	    else
	       begin
	          qi=di;
	       end
	 end
   end
   endtask

   task write_mem;
      input [addr_width-1:0] a;
      input [BITS-1:0] d;
 
   begin
      casez({valid_address(a)})
	1'b0: 
		x_mem;
	1'b1: mem[a]=d;
      endcase
   end
   endtask

   task write_mem_x;
      input [addr_width-1:0] a;
   begin
      casez({valid_address(a)})
	1'b0: 
		x_mem;
	1'b1: mem[a]=wordx;
      endcase
   end
   endtask

   task x_mem;
      integer n;
   begin
      for (n=0; n<word_depth; n=n+1)
	 mem[n]=wordx;
   end
   endtask

   task process_violations;
   begin
      if ((NOT_clk_PER!==LAST_NOT_clk_PER) ||
	  (NOT_clk_MINH!==LAST_NOT_clk_MINH) ||
	  (NOT_clk_MINL!==LAST_NOT_clk_MINL))
	 begin
	    if (ceni !== 1'b1)
               begin
		  x_mem;
		  read_mem(0,1);
	       end
	 end
      else
	 begin
	    update_notifier_buses;
	    x_inputs;
	    update_logic;
	    mem_cycle;
	 end
      update_last_notifiers;
   end
   endtask

   function valid_address;
      input [addr_width-1:0] a;
   begin
      valid_address = (^(a) !== 1'bx);
   end
   endfunction


   buf (q[0], _q[0]);
   buf (q[1], _q[1]);
   buf (q[2], _q[2]);
   buf (q[3], _q[3]);
   buf (q[4], _q[4]);
   buf (q[5], _q[5]);
   buf (q[6], _q[6]);
   buf (q[7], _q[7]);
   buf (q[8], _q[8]);
   buf (q[9], _q[9]);
   buf (q[10], _q[10]);
   buf (q[11], _q[11]);
   buf (q[12], _q[12]);
   buf (q[13], _q[13]);
   buf (q[14], _q[14]);
   buf (q[15], _q[15]);
   buf (q[16], _q[16]);
   buf (q[17], _q[17]);
   buf (q[18], _q[18]);
   buf (q[19], _q[19]);
   buf (q[20], _q[20]);
   buf (q[21], _q[21]);
   buf (q[22], _q[22]);
   buf (q[23], _q[23]);
   buf (q[24], _q[24]);
   buf (q[25], _q[25]);
   buf (q[26], _q[26]);
   buf (q[27], _q[27]);
   buf (q[28], _q[28]);
   buf (q[29], _q[29]);
   buf (q[30], _q[30]);
   buf (q[31], _q[31]);
   buf (_d[0], d[0]);
   buf (_d[1], d[1]);
   buf (_d[2], d[2]);
   buf (_d[3], d[3]);
   buf (_d[4], d[4]);
   buf (_d[5], d[5]);
   buf (_d[6], d[6]);
   buf (_d[7], d[7]);
   buf (_d[8], d[8]);
   buf (_d[9], d[9]);
   buf (_d[10], d[10]);
   buf (_d[11], d[11]);
   buf (_d[12], d[12]);
   buf (_d[13], d[13]);
   buf (_d[14], d[14]);
   buf (_d[15], d[15]);
   buf (_d[16], d[16]);
   buf (_d[17], d[17]);
   buf (_d[18], d[18]);
   buf (_d[19], d[19]);
   buf (_d[20], d[20]);
   buf (_d[21], d[21]);
   buf (_d[22], d[22]);
   buf (_d[23], d[23]);
   buf (_d[24], d[24]);
   buf (_d[25], d[25]);
   buf (_d[26], d[26]);
   buf (_d[27], d[27]);
   buf (_d[28], d[28]);
   buf (_d[29], d[29]);
   buf (_d[30], d[30]);
   buf (_d[31], d[31]);
   buf (_a[0], a[0]);
   buf (_a[1], a[1]);
   buf (_a[2], a[2]);
   buf (_a[3], a[3]);
   buf (_a[4], a[4]);
   buf (_a[5], a[5]);
   buf (_a[6], a[6]);
   buf (_a[7], a[7]);
   buf (_a[8], a[8]);
   buf (_a[9], a[9]);
   buf (_a[10], a[10]);
   buf (_clk, clk);
   buf (_wen, wen);
   buf (_cen, cen);


   assign _q = qi;
   assign re_flag = !(_cen);
   assign re_data_flag = !(_cen || _wen);


   always @(
	    NOT_a0 or
	    NOT_a1 or
	    NOT_a2 or
	    NOT_a3 or
	    NOT_a4 or
	    NOT_a5 or
	    NOT_a6 or
	    NOT_a7 or
	    NOT_a8 or
	    NOT_a9 or
	    NOT_a10 or
	    NOT_d0 or
	    NOT_d1 or
	    NOT_d2 or
	    NOT_d3 or
	    NOT_d4 or
	    NOT_d5 or
	    NOT_d6 or
	    NOT_d7 or
	    NOT_d8 or
	    NOT_d9 or
	    NOT_d10 or
	    NOT_d11 or
	    NOT_d12 or
	    NOT_d13 or
	    NOT_d14 or
	    NOT_d15 or
	    NOT_d16 or
	    NOT_d17 or
	    NOT_d18 or
	    NOT_d19 or
	    NOT_d20 or
	    NOT_d21 or
	    NOT_d22 or
	    NOT_d23 or
	    NOT_d24 or
	    NOT_d25 or
	    NOT_d26 or
	    NOT_d27 or
	    NOT_d28 or
	    NOT_d29 or
	    NOT_d30 or
	    NOT_d31 or
	    NOT_wen or
	    NOT_cen or
	    NOT_clk_PER or
	    NOT_clk_MINH or
	    NOT_clk_MINL
	    )
      begin
         process_violations;
      end

   always @( _clk )
      begin
         casez({LAST_clk,_clk})
	   2'b01: begin
	      latch_inputs;
	      update_logic;
	      mem_cycle;
	   end

	   2'b10,
	   2'bx?,
	   2'b00,
	   2'b11: ;

	   2'b?x: begin
	      x_mem;
              read_mem(0,1);
	   end
	   
	 endcase
	 LAST_clk = _clk;
      end

   specify
      $setuphold(posedge clk, cen, 1.000, 0.500, NOT_cen);
      $setuphold(posedge clk &&& re_flag, wen, 1.000, 0.500, NOT_wen);
      $setuphold(posedge clk &&& re_flag, a[0], 1.000, 0.500, NOT_a0);
      $setuphold(posedge clk &&& re_flag, a[1], 1.000, 0.500, NOT_a1);
      $setuphold(posedge clk &&& re_flag, a[2], 1.000, 0.500, NOT_a2);
      $setuphold(posedge clk &&& re_flag, a[3], 1.000, 0.500, NOT_a3);
      $setuphold(posedge clk &&& re_flag, a[4], 1.000, 0.500, NOT_a4);
      $setuphold(posedge clk &&& re_flag, a[5], 1.000, 0.500, NOT_a5);
      $setuphold(posedge clk &&& re_flag, a[6], 1.000, 0.500, NOT_a6);
      $setuphold(posedge clk &&& re_flag, a[7], 1.000, 0.500, NOT_a7);
      $setuphold(posedge clk &&& re_flag, a[8], 1.000, 0.500, NOT_a8);
      $setuphold(posedge clk &&& re_flag, a[9], 1.000, 0.500, NOT_a9);
      $setuphold(posedge clk &&& re_flag, a[10], 1.000, 0.500, NOT_a10);
      $setuphold(posedge clk &&& re_data_flag, d[0], 1.000, 0.500, NOT_d0);
      $setuphold(posedge clk &&& re_data_flag, d[1], 1.000, 0.500, NOT_d1);
      $setuphold(posedge clk &&& re_data_flag, d[2], 1.000, 0.500, NOT_d2);
      $setuphold(posedge clk &&& re_data_flag, d[3], 1.000, 0.500, NOT_d3);
      $setuphold(posedge clk &&& re_data_flag, d[4], 1.000, 0.500, NOT_d4);
      $setuphold(posedge clk &&& re_data_flag, d[5], 1.000, 0.500, NOT_d5);
      $setuphold(posedge clk &&& re_data_flag, d[6], 1.000, 0.500, NOT_d6);
      $setuphold(posedge clk &&& re_data_flag, d[7], 1.000, 0.500, NOT_d7);
      $setuphold(posedge clk &&& re_data_flag, d[8], 1.000, 0.500, NOT_d8);
      $setuphold(posedge clk &&& re_data_flag, d[9], 1.000, 0.500, NOT_d9);
      $setuphold(posedge clk &&& re_data_flag, d[10], 1.000, 0.500, NOT_d10);
      $setuphold(posedge clk &&& re_data_flag, d[11], 1.000, 0.500, NOT_d11);
      $setuphold(posedge clk &&& re_data_flag, d[12], 1.000, 0.500, NOT_d12);
      $setuphold(posedge clk &&& re_data_flag, d[13], 1.000, 0.500, NOT_d13);
      $setuphold(posedge clk &&& re_data_flag, d[14], 1.000, 0.500, NOT_d14);
      $setuphold(posedge clk &&& re_data_flag, d[15], 1.000, 0.500, NOT_d15);
      $setuphold(posedge clk &&& re_data_flag, d[16], 1.000, 0.500, NOT_d16);
      $setuphold(posedge clk &&& re_data_flag, d[17], 1.000, 0.500, NOT_d17);
      $setuphold(posedge clk &&& re_data_flag, d[18], 1.000, 0.500, NOT_d18);
      $setuphold(posedge clk &&& re_data_flag, d[19], 1.000, 0.500, NOT_d19);
      $setuphold(posedge clk &&& re_data_flag, d[20], 1.000, 0.500, NOT_d20);
      $setuphold(posedge clk &&& re_data_flag, d[21], 1.000, 0.500, NOT_d21);
      $setuphold(posedge clk &&& re_data_flag, d[22], 1.000, 0.500, NOT_d22);
      $setuphold(posedge clk &&& re_data_flag, d[23], 1.000, 0.500, NOT_d23);
      $setuphold(posedge clk &&& re_data_flag, d[24], 1.000, 0.500, NOT_d24);
      $setuphold(posedge clk &&& re_data_flag, d[25], 1.000, 0.500, NOT_d25);
      $setuphold(posedge clk &&& re_data_flag, d[26], 1.000, 0.500, NOT_d26);
      $setuphold(posedge clk &&& re_data_flag, d[27], 1.000, 0.500, NOT_d27);
      $setuphold(posedge clk &&& re_data_flag, d[28], 1.000, 0.500, NOT_d28);
      $setuphold(posedge clk &&& re_data_flag, d[29], 1.000, 0.500, NOT_d29);
      $setuphold(posedge clk &&& re_data_flag, d[30], 1.000, 0.500, NOT_d30);
      $setuphold(posedge clk &&& re_data_flag, d[31], 1.000, 0.500, NOT_d31);

      $period(posedge clk, 3.000, NOT_clk_PER);
      $width(posedge clk, 1.000, 0, NOT_clk_MINH);
      $width(negedge clk, 1.000, 0, NOT_clk_MINL);

      (clk => q[0])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[1])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[2])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[3])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[4])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[5])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[6])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[7])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[8])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[9])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[10])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[11])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[12])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[13])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[14])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[15])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[16])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[17])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[18])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[19])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[20])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[21])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[22])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[23])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[24])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[25])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[26])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[27])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[28])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[29])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[30])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (clk => q[31])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
   endspecify

endmodule
`endcelldefine
