//
//      CONFIDENTIAL AND PROPRIETARY SOFTWARE/DATA OF ARTISAN COMPONENTS, INC.
//      
//      Copyright (c) 2000 Artisan Components, Inc.  All Rights Reserved.
//      
//      Use of this Software/Data is subject to the terms and conditions of
//      the applicable license agreement between Artisan Components, Inc. and
//      Taiwan Semiconductor Manufacturing Ltd..  In addition, this Software/Data
//      is protected by copyright law and international treaties.
//      
//      The copyright notice(s) in this Software/Data does not indicate actual
//      or intended publication of this Software/Data.
//
//      Verilog model for Synchronous Single-Port Ram With UTI
//
//      Instance Name:  art_hssp_128x34
//      Words:          128
//      Word Width:     34
//      Pipeline:       No
//
//      Creation Date:  2000-12-07 08:15:07Z
//      Version: 	99Q1V0
//
//      Verified With: Cadence Verilog-XL
//
//      Modeling Assumptions: This model supports full gate level simulation
//          including proper x-handling and timing check behavior.  Unit
//          delay timing is included in the model. Back-annotation of SDF
//          (v2.1) is supported.  SDF can be created utilyzing the delay
//          calculation views provided with this generator and supported
//          delay calculators.  All buses are modeled [MSB:LSB].  All 
//          ports are padded with Verilog primitives.
//
//      Modeling Limitations: The output hold function has been deleted
//          completely from this model.  Most Verilog simulators are 
//          incapable of scheduling more than 1 event on the rising 
//          edge of clock.  Therefore, it is impossible to model
//          the output hold (to x) action correctly.  It is necessary
//          to run static path timing tools using Artisan supplied
//          timing models to insure that the output hold time is
//          sufficient enough to not violate hold time constraints
//          of downstream flip-flops.
//
//      Known Bugs: None.
//
//      Known Work Arounds: N/A
//
`timescale 1 ns/10 ps
`celldefine

module art_hssp_128x34 (
   q,
   censq,
   oensq,
   wensq,
   asq,
   tq,
   clk,
   cen,
   oen,
   wen,
   a,
   d,
   tis,
   tms,
   tcen,
   toen,
   tqoen,
   twen,
   ta,
   td
);
   parameter		   BITS = 34;
   parameter		   word_depth = 128;
   parameter		   addr_width = 7;
   parameter		   wordx = {BITS{1'bx}};
   parameter		   addrx = {addr_width{1'bx}};
	
   output [33:0] q;
   output censq;
   output oensq;
   output wensq;
   output [6:0] asq;
   output [33:0] tq;
   input clk;
   input cen;
   input oen;
   input wen;
   input [6:0] a;
   input [33:0] d;
   input tis;
   input tms;
   input tcen;
   input toen;
   input tqoen;
   input twen;
   input [6:0] ta;
   input [33:0] td;

   reg [BITS-1:0]	   mem [word_depth-1:0];

   reg			   NOT_cen;
   reg			   NOT_oen;
   reg			   NOT_wen;
   reg			   NOT_a0;
   reg			   NOT_a1;
   reg			   NOT_a2;
   reg			   NOT_a3;
   reg			   NOT_a4;
   reg			   NOT_a5;
   reg			   NOT_a6;
   reg [addr_width-1:0]	   NOT_a;
   reg			   NOT_d0;
   reg			   NOT_d1;
   reg			   NOT_d2;
   reg			   NOT_d3;
   reg			   NOT_d4;
   reg			   NOT_d5;
   reg			   NOT_d6;
   reg			   NOT_d7;
   reg			   NOT_d8;
   reg			   NOT_d9;
   reg			   NOT_d10;
   reg			   NOT_d11;
   reg			   NOT_d12;
   reg			   NOT_d13;
   reg			   NOT_d14;
   reg			   NOT_d15;
   reg			   NOT_d16;
   reg			   NOT_d17;
   reg			   NOT_d18;
   reg			   NOT_d19;
   reg			   NOT_d20;
   reg			   NOT_d21;
   reg			   NOT_d22;
   reg			   NOT_d23;
   reg			   NOT_d24;
   reg			   NOT_d25;
   reg			   NOT_d26;
   reg			   NOT_d27;
   reg			   NOT_d28;
   reg			   NOT_d29;
   reg			   NOT_d30;
   reg			   NOT_d31;
   reg			   NOT_d32;
   reg			   NOT_d33;
   reg [BITS-1:0]	   NOT_d;
   reg			   NOT_tis;
   reg			   NOT_tms;
   reg			   NOT_tcen;
   reg			   NOT_toen;
   reg			   NOT_twen;
   reg			   NOT_ta0;
   reg			   NOT_ta1;
   reg			   NOT_ta2;
   reg			   NOT_ta3;
   reg			   NOT_ta4;
   reg			   NOT_ta5;
   reg			   NOT_ta6;
   reg [addr_width-1:0]	   NOT_ta;
   reg			   NOT_td0;
   reg			   NOT_td1;
   reg			   NOT_td2;
   reg			   NOT_td3;
   reg			   NOT_td4;
   reg			   NOT_td5;
   reg			   NOT_td6;
   reg			   NOT_td7;
   reg			   NOT_td8;
   reg			   NOT_td9;
   reg			   NOT_td10;
   reg			   NOT_td11;
   reg			   NOT_td12;
   reg			   NOT_td13;
   reg			   NOT_td14;
   reg			   NOT_td15;
   reg			   NOT_td16;
   reg			   NOT_td17;
   reg			   NOT_td18;
   reg			   NOT_td19;
   reg			   NOT_td20;
   reg			   NOT_td21;
   reg			   NOT_td22;
   reg			   NOT_td23;
   reg			   NOT_td24;
   reg			   NOT_td25;
   reg			   NOT_td26;
   reg			   NOT_td27;
   reg			   NOT_td28;
   reg			   NOT_td29;
   reg			   NOT_td30;
   reg			   NOT_td31;
   reg			   NOT_td32;
   reg			   NOT_td33;
   reg [BITS-1:0]	   NOT_td;
   reg			   NOT_clk_PER;
   reg			   NOT_clk_MINH;
   reg			   NOT_clk_MINL;

   reg			   LAST_NOT_cen;
   reg			   LAST_NOT_oen;
   reg			   LAST_NOT_wen;
   reg [addr_width-1:0]	   LAST_NOT_a;
   reg [BITS-1:0]	   LAST_NOT_d;
   reg			   LAST_NOT_tis;
   reg			   LAST_NOT_tms;
   reg			   LAST_NOT_tcen;
   reg			   LAST_NOT_toen;
   reg			   LAST_NOT_twen;
   reg [addr_width-1:0]	   LAST_NOT_ta;
   reg [BITS-1:0]	   LAST_NOT_td;
   reg			   LAST_NOT_clk_PER;
   reg			   LAST_NOT_clk_MINH;
   reg			   LAST_NOT_clk_MINL;


   wire [BITS-1:0]   _q;
   wire [addr_width-1:0]   _asq;
   wire			   _wensq;
   wire			   _oensq;
   wire			   _censq;
   wire [BITS-1:0]   _tq;
   wire			   _oeni;
   wire [addr_width-1:0]   _a;
   wire			   _clk;
   wire			   _cen;
   wire			   _oen;
   wire			   _wen;
   wire [BITS-1:0]   _d;
   wire			   _tis;
   wire [addr_width-1:0]   _ta;
   wire			   _toeni;
   wire			   _twen;
   wire			   _toen;
   wire			   _tms;
   wire			   _tcen;
   wire			   _tqoen;
   wire [BITS-1:0]   _td;
   wire                    re_flag;
   wire                    re_data_flag;

   reg			   LATCHED_cen;
   reg			   LATCHED_wen;
   reg [addr_width-1:0]	   LATCHED_a;
   reg [BITS-1:0]	   LATCHED_d;
   reg			   LATCHED_tis;
   reg			   LATCHED_oen;
   reg			   LATCHED_twen;
   reg			   LATCHED_toen;
   reg			   LATCHED_tms;
   reg			   LATCHED_tcen;
   reg [addr_width-1:0]	   LATCHED_ta;
   reg [BITS-1:0]	   LATCHED_td;

   reg			   ceni;
   reg			   weni;
   reg [addr_width-1:0]	   ai;
   reg [BITS-1:0]	   di;
   reg [BITS-1:0]	   qi;
   reg [BITS-1:0]	   LAST_qi;

   reg			   cen_ffin;
   reg			   oen_ffin;
   reg			   wen_ffin;
   reg [BITS-1:0]	   d_ffin;

   reg			   censq_reg;
   reg			   oensq_reg;
   reg			   wensq_reg;
   reg [addr_width-1:0]	   asq_reg;
   reg [BITS-1:0]	   d_reg;

   reg			   LAST_clk;

   reg                     valid_cycle;


   task update_notifier_buses;
   begin
      NOT_a = {
               NOT_a6,
               NOT_a5,
               NOT_a4,
               NOT_a3,
               NOT_a2,
               NOT_a1,
               NOT_a0};
      NOT_d = {
               NOT_d33,
               NOT_d32,
               NOT_d31,
               NOT_d30,
               NOT_d29,
               NOT_d28,
               NOT_d27,
               NOT_d26,
               NOT_d25,
               NOT_d24,
               NOT_d23,
               NOT_d22,
               NOT_d21,
               NOT_d20,
               NOT_d19,
               NOT_d18,
               NOT_d17,
               NOT_d16,
               NOT_d15,
               NOT_d14,
               NOT_d13,
               NOT_d12,
               NOT_d11,
               NOT_d10,
               NOT_d9,
               NOT_d8,
               NOT_d7,
               NOT_d6,
               NOT_d5,
               NOT_d4,
               NOT_d3,
               NOT_d2,
               NOT_d1,
               NOT_d0};
      NOT_ta = {
                NOT_ta6,
                NOT_ta5,
                NOT_ta4,
                NOT_ta3,
                NOT_ta2,
                NOT_ta1,
                NOT_ta0};
      NOT_td = {
                NOT_td33,
                NOT_td32,
                NOT_td31,
                NOT_td30,
                NOT_td29,
                NOT_td28,
                NOT_td27,
                NOT_td26,
                NOT_td25,
                NOT_td24,
                NOT_td23,
                NOT_td22,
                NOT_td21,
                NOT_td20,
                NOT_td19,
                NOT_td18,
                NOT_td17,
                NOT_td16,
                NOT_td15,
                NOT_td14,
                NOT_td13,
                NOT_td12,
                NOT_td11,
                NOT_td10,
                NOT_td9,
                NOT_td8,
                NOT_td7,
                NOT_td6,
                NOT_td5,
                NOT_td4,
                NOT_td3,
                NOT_td2,
                NOT_td1,
                NOT_td0};
   end
   endtask

   task mem_cycle;
   begin
      valid_cycle = 1'bx;
      casez({weni,ceni})
	2'b10: begin
	   valid_cycle = 1;
	   update_flops(0);
	   read_mem(1,0);
	end
	2'b00: begin
	   valid_cycle = 0;
	   update_flops(0);
	   write_mem(ai,di);
	   read_mem(0,0);
	end
	2'b?1: ;
	2'b1x: begin
	   valid_cycle = 1;
	   update_flops(1);
	   read_mem(0,1);
	end
	2'bx0: begin
	   valid_cycle = 0;
	   update_flops(0);
	   write_mem_x(ai);
	   read_mem(0,1);
	end
	2'b0x,
	2'bxx: begin
	   valid_cycle = 0;
	   update_flops(1);
	   write_mem_x(ai);
	   read_mem(0,1);
	end
      endcase
   end
   endtask
      

   task update_last_notifiers;
   begin
      LAST_NOT_a = NOT_a;
      LAST_NOT_d = NOT_d;
      LAST_NOT_wen = NOT_wen;
      LAST_NOT_oen = NOT_oen;
      LAST_NOT_cen = NOT_cen;
      LAST_NOT_tis = NOT_tis;
      LAST_NOT_twen = NOT_twen;
      LAST_NOT_toen = NOT_toen;
      LAST_NOT_tms = NOT_tms;
      LAST_NOT_tcen = NOT_tcen;
      LAST_NOT_ta = NOT_ta;
      LAST_NOT_td = NOT_td;
      LAST_NOT_clk_PER = NOT_clk_PER;
      LAST_NOT_clk_MINH = NOT_clk_MINH;
      LAST_NOT_clk_MINL = NOT_clk_MINL;
   end
   endtask

   task latch_inputs;
   begin
      LATCHED_a = _a ;
      LATCHED_d = _d ;
      LATCHED_wen = _wen ;
      LATCHED_cen = _cen ;
      LATCHED_tis = _tis ;
      LATCHED_oen = _oen ;
      LATCHED_twen = _twen ;
      LATCHED_toen = _toen ;
      LATCHED_tms = _tms ;
      LATCHED_tcen = _tcen ;
      LATCHED_ta = _ta ;
      LATCHED_td = _td ;
      LAST_qi = qi;
   end
   endtask


   task update_logic;
   begin
      ceni = !(LATCHED_tms || !(!LATCHED_tis ? LATCHED_cen : LATCHED_tcen));
      weni = !(!LATCHED_tms && !(!LATCHED_tis ? LATCHED_wen : LATCHED_twen));
      ai = (!LATCHED_tis ? LATCHED_a : LATCHED_ta) ;
      di = (!LATCHED_tis ? LATCHED_d : LATCHED_td);
      cen_ffin = (!LATCHED_tis ? LATCHED_cen : LATCHED_tcen);
      oen_ffin = (!LATCHED_tis ? LATCHED_oen : LATCHED_toen);
      wen_ffin = (!LATCHED_tis ? LATCHED_wen : LATCHED_twen);
      d_ffin = (!LATCHED_tms ? LAST_qi : (!LATCHED_tis ? LATCHED_d : LATCHED_td));
   end
   endtask


   task update_flops;
      input xflag;
   begin
      asq_reg = xflag ? addrx : ai;
      d_reg = xflag ? wordx : d_ffin ;
      oensq_reg = xflag ? 1'bx : oen_ffin ;
      wensq_reg = xflag ? 1'bx : wen_ffin ;
      censq_reg = xflag ? 1'bx : cen_ffin ;
   end
   endtask

   task x_inputs;
      integer n;
   begin
      for (n=0; n<addr_width; n=n+1)
	 begin
	    LATCHED_a[n] = (NOT_a[n]!==LAST_NOT_a[n]) ? 1'bx : LATCHED_a[n] ;
	    LATCHED_ta[n] = (NOT_ta[n]!==LAST_NOT_ta[n]) ? 1'bx : LATCHED_ta[n] ;
	 end
      for (n=0; n<BITS; n=n+1)
	 begin
	    LATCHED_d[n] = (NOT_d[n]!==LAST_NOT_d[n]) ? 1'bx : LATCHED_d[n] ;
	    LATCHED_td[n] = (NOT_td[n]!==LAST_NOT_td[n]) ? 1'bx : LATCHED_td[n] ;
	 end
      LATCHED_wen = (NOT_wen!==LAST_NOT_wen) ? 1'bx : LATCHED_wen ;
      LATCHED_oen = (NOT_oen!==LAST_NOT_oen) ? 1'bx : LATCHED_oen ;
      LATCHED_cen = (NOT_cen!==LAST_NOT_cen) ? 1'bx : LATCHED_cen ;
      LATCHED_tis = (NOT_tis!==LAST_NOT_tis) ? 1'bx : LATCHED_tis ;
      LATCHED_twen = (NOT_twen!==LAST_NOT_twen) ? 1'bx : LATCHED_twen ;
      LATCHED_toen = (NOT_toen!==LAST_NOT_toen) ? 1'bx : LATCHED_toen ;
      LATCHED_tms = (NOT_tms!==LAST_NOT_tms) ? 1'bx : LATCHED_tms ;
      LATCHED_tcen = (NOT_tcen!==LAST_NOT_tcen) ? 1'bx : LATCHED_tcen ;
   end
   endtask

   task read_mem;
      input r_wb;
      input xflag;
   begin
      if (r_wb)
	 begin
	    if (valid_address(ai))
	       begin
	          qi=mem[ai];
	       end
	    else
	       begin
		  qi=wordx;
	       end
	 end
      else
	 begin
	    if (xflag)
	       begin
		  qi=wordx;
	       end
	    else
	       begin
	          qi=di;
	       end
	 end
   end
   endtask

   task write_mem;
      input [addr_width-1:0] a;
      input [BITS-1:0] d;
   begin
      casez({valid_address(a)})
	1'b0: x_mem;
	1'b1: mem[a]=d;
      endcase
   end
   endtask

   task write_mem_x;
      input [addr_width-1:0] a;
   begin
      casez({valid_address(a)})
	1'b0: x_mem;
	1'b1: mem[a]=wordx;
      endcase
   end
   endtask

   task x_mem;
      integer n;
   begin
      for (n=0; n<word_depth; n=n+1)
	 mem[n]=wordx;
   end
   endtask

   function valid_address;
      input [addr_width-1:0] a;
   begin
      valid_address = (^(a) !== 1'bx);
   end
   endfunction

   buf (asq[0], _asq[0]);
   buf (asq[1], _asq[1]);
   buf (asq[2], _asq[2]);
   buf (asq[3], _asq[3]);
   buf (asq[4], _asq[4]);
   buf (asq[5], _asq[5]);
   buf (asq[6], _asq[6]);
   buf (wensq, _wensq);
   buf (oensq, _oensq);
   buf (censq, _censq);
   bufif0 (q[0], _q[0], _oeni);
   bufif0 (q[1], _q[1], _oeni);
   bufif0 (q[2], _q[2], _oeni);
   bufif0 (q[3], _q[3], _oeni);
   bufif0 (q[4], _q[4], _oeni);
   bufif0 (q[5], _q[5], _oeni);
   bufif0 (q[6], _q[6], _oeni);
   bufif0 (q[7], _q[7], _oeni);
   bufif0 (q[8], _q[8], _oeni);
   bufif0 (q[9], _q[9], _oeni);
   bufif0 (q[10], _q[10], _oeni);
   bufif0 (q[11], _q[11], _oeni);
   bufif0 (q[12], _q[12], _oeni);
   bufif0 (q[13], _q[13], _oeni);
   bufif0 (q[14], _q[14], _oeni);
   bufif0 (q[15], _q[15], _oeni);
   bufif0 (q[16], _q[16], _oeni);
   bufif0 (q[17], _q[17], _oeni);
   bufif0 (q[18], _q[18], _oeni);
   bufif0 (q[19], _q[19], _oeni);
   bufif0 (q[20], _q[20], _oeni);
   bufif0 (q[21], _q[21], _oeni);
   bufif0 (q[22], _q[22], _oeni);
   bufif0 (q[23], _q[23], _oeni);
   bufif0 (q[24], _q[24], _oeni);
   bufif0 (q[25], _q[25], _oeni);
   bufif0 (q[26], _q[26], _oeni);
   bufif0 (q[27], _q[27], _oeni);
   bufif0 (q[28], _q[28], _oeni);
   bufif0 (q[29], _q[29], _oeni);
   bufif0 (q[30], _q[30], _oeni);
   bufif0 (q[31], _q[31], _oeni);
   bufif0 (q[32], _q[32], _oeni);
   bufif0 (q[33], _q[33], _oeni);
   bufif1 (tq[0], _tq[0],_toeni);
   bufif1 (tq[1], _tq[1],_toeni);
   bufif1 (tq[2], _tq[2],_toeni);
   bufif1 (tq[3], _tq[3],_toeni);
   bufif1 (tq[4], _tq[4],_toeni);
   bufif1 (tq[5], _tq[5],_toeni);
   bufif1 (tq[6], _tq[6],_toeni);
   bufif1 (tq[7], _tq[7],_toeni);
   bufif1 (tq[8], _tq[8],_toeni);
   bufif1 (tq[9], _tq[9],_toeni);
   bufif1 (tq[10], _tq[10],_toeni);
   bufif1 (tq[11], _tq[11],_toeni);
   bufif1 (tq[12], _tq[12],_toeni);
   bufif1 (tq[13], _tq[13],_toeni);
   bufif1 (tq[14], _tq[14],_toeni);
   bufif1 (tq[15], _tq[15],_toeni);
   bufif1 (tq[16], _tq[16],_toeni);
   bufif1 (tq[17], _tq[17],_toeni);
   bufif1 (tq[18], _tq[18],_toeni);
   bufif1 (tq[19], _tq[19],_toeni);
   bufif1 (tq[20], _tq[20],_toeni);
   bufif1 (tq[21], _tq[21],_toeni);
   bufif1 (tq[22], _tq[22],_toeni);
   bufif1 (tq[23], _tq[23],_toeni);
   bufif1 (tq[24], _tq[24],_toeni);
   bufif1 (tq[25], _tq[25],_toeni);
   bufif1 (tq[26], _tq[26],_toeni);
   bufif1 (tq[27], _tq[27],_toeni);
   bufif1 (tq[28], _tq[28],_toeni);
   bufif1 (tq[29], _tq[29],_toeni);
   bufif1 (tq[30], _tq[30],_toeni);
   bufif1 (tq[31], _tq[31],_toeni);
   bufif1 (tq[32], _tq[32],_toeni);
   bufif1 (tq[33], _tq[33],_toeni);
   buf (_d[0], d[0]);
   buf (_d[1], d[1]);
   buf (_d[2], d[2]);
   buf (_d[3], d[3]);
   buf (_d[4], d[4]);
   buf (_d[5], d[5]);
   buf (_d[6], d[6]);
   buf (_d[7], d[7]);
   buf (_d[8], d[8]);
   buf (_d[9], d[9]);
   buf (_d[10], d[10]);
   buf (_d[11], d[11]);
   buf (_d[12], d[12]);
   buf (_d[13], d[13]);
   buf (_d[14], d[14]);
   buf (_d[15], d[15]);
   buf (_d[16], d[16]);
   buf (_d[17], d[17]);
   buf (_d[18], d[18]);
   buf (_d[19], d[19]);
   buf (_d[20], d[20]);
   buf (_d[21], d[21]);
   buf (_d[22], d[22]);
   buf (_d[23], d[23]);
   buf (_d[24], d[24]);
   buf (_d[25], d[25]);
   buf (_d[26], d[26]);
   buf (_d[27], d[27]);
   buf (_d[28], d[28]);
   buf (_d[29], d[29]);
   buf (_d[30], d[30]);
   buf (_d[31], d[31]);
   buf (_d[32], d[32]);
   buf (_d[33], d[33]);
   buf (_a[0], a[0]);
   buf (_a[1], a[1]);
   buf (_a[2], a[2]);
   buf (_a[3], a[3]);
   buf (_a[4], a[4]);
   buf (_a[5], a[5]);
   buf (_a[6], a[6]);
   buf (_clk, clk);
   buf (_wen, wen);
   buf (_oen, oen);
   buf (_cen, cen);
   buf (_tis, tis);
   buf (_twen, twen);
   buf (_toen, toen);
   buf (_tms, tms);
   buf (_tcen, tcen);
   buf (_tqoen, tqoen);
   buf (_ta[0], ta[0]);
   buf (_ta[1], ta[1]);
   buf (_ta[2], ta[2]);
   buf (_ta[3], ta[3]);
   buf (_ta[4], ta[4]);
   buf (_ta[5], ta[5]);
   buf (_ta[6], ta[6]);
   buf (_td[0], td[0]);
   buf (_td[1], td[1]);
   buf (_td[2], td[2]);
   buf (_td[3], td[3]);
   buf (_td[4], td[4]);
   buf (_td[5], td[5]);
   buf (_td[6], td[6]);
   buf (_td[7], td[7]);
   buf (_td[8], td[8]);
   buf (_td[9], td[9]);
   buf (_td[10], td[10]);
   buf (_td[11], td[11]);
   buf (_td[12], td[12]);
   buf (_td[13], td[13]);
   buf (_td[14], td[14]);
   buf (_td[15], td[15]);
   buf (_td[16], td[16]);
   buf (_td[17], td[17]);
   buf (_td[18], td[18]);
   buf (_td[19], td[19]);
   buf (_td[20], td[20]);
   buf (_td[21], td[21]);
   buf (_td[22], td[22]);
   buf (_td[23], td[23]);
   buf (_td[24], td[24]);
   buf (_td[25], td[25]);
   buf (_td[26], td[26]);
   buf (_td[27], td[27]);
   buf (_td[28], td[28]);
   buf (_td[29], td[29]);
   buf (_td[30], td[30]);
   buf (_td[31], td[31]);
   buf (_td[32], td[32]);
   buf (_td[33], td[33]);

   assign _oeni = (_tms ? _oen : ( _tis ? _toen : _oen) );
   assign _toeni = ~_tqoen ;
   assign _oensq = oensq_reg;
   assign _wensq = wensq_reg;
   assign _censq = censq_reg;
   assign _asq = asq_reg;
   assign _tq = d_reg;
   assign _q = _tms ? d_reg : qi;
   assign re_flag = (_tms || !( _tis ? _tcen : _cen));
   assign re_data_flag = ( _tms || (!( _tis ? _tcen : _cen) && 
      !( _tis ? _twen : _wen)));


   always @(
	    NOT_a0 or
	    NOT_a1 or
	    NOT_a2 or
	    NOT_a3 or
	    NOT_a4 or
	    NOT_a5 or
	    NOT_a6 or
	    NOT_d0 or
	    NOT_d1 or
	    NOT_d2 or
	    NOT_d3 or
	    NOT_d4 or
	    NOT_d5 or
	    NOT_d6 or
	    NOT_d7 or
	    NOT_d8 or
	    NOT_d9 or
	    NOT_d10 or
	    NOT_d11 or
	    NOT_d12 or
	    NOT_d13 or
	    NOT_d14 or
	    NOT_d15 or
	    NOT_d16 or
	    NOT_d17 or
	    NOT_d18 or
	    NOT_d19 or
	    NOT_d20 or
	    NOT_d21 or
	    NOT_d22 or
	    NOT_d23 or
	    NOT_d24 or
	    NOT_d25 or
	    NOT_d26 or
	    NOT_d27 or
	    NOT_d28 or
	    NOT_d29 or
	    NOT_d30 or
	    NOT_d31 or
	    NOT_d32 or
	    NOT_d33 or
	    NOT_wen or
	    NOT_oen or
	    NOT_cen or
	    NOT_tis or
	    NOT_twen or
	    NOT_toen or
	    NOT_tms or
	    NOT_tcen or
	    NOT_ta0 or
	    NOT_ta1 or
	    NOT_ta2 or
	    NOT_ta3 or
	    NOT_ta4 or
	    NOT_ta5 or
	    NOT_ta6 or
	    NOT_td0 or
	    NOT_td1 or
	    NOT_td2 or
	    NOT_td3 or
	    NOT_td4 or
	    NOT_td5 or
	    NOT_td6 or
	    NOT_td7 or
	    NOT_td8 or
	    NOT_td9 or
	    NOT_td10 or
	    NOT_td11 or
	    NOT_td12 or
	    NOT_td13 or
	    NOT_td14 or
	    NOT_td15 or
	    NOT_td16 or
	    NOT_td17 or
	    NOT_td18 or
	    NOT_td19 or
	    NOT_td20 or
	    NOT_td21 or
	    NOT_td22 or
	    NOT_td23 or
	    NOT_td24 or
	    NOT_td25 or
	    NOT_td26 or
	    NOT_td27 or
	    NOT_td28 or
	    NOT_td29 or
	    NOT_td30 or
	    NOT_td31 or
	    NOT_td32 or
	    NOT_td33 or
	    NOT_clk_PER or
	    NOT_clk_MINH or
	    NOT_clk_MINL
	    )
      begin
	 if ((NOT_clk_PER!==LAST_NOT_clk_PER) ||
	     (NOT_clk_MINH!==LAST_NOT_clk_MINH) ||
	     (NOT_clk_MINL!==LAST_NOT_clk_MINL))
	    begin
	       if (ceni !== 1'b1)
		  begin
		     update_flops(1);
		     x_mem;
		     read_mem(0,1);
		  end
	    end
	 else
	    begin
	       update_notifier_buses;
	       x_inputs;
	       update_logic;
	       mem_cycle;
	    end
	 update_last_notifiers;
      end

   always @( _clk )
      begin
         casez({LAST_clk,_clk})
	   2'b01: begin
	      latch_inputs;
	      update_logic;
	      mem_cycle;
	   end

	   2'b10,
	   2'bx?,
	   2'b00,
	   2'b11: ;

	   2'b?x: begin
	      update_flops(1);
	      x_mem;
              read_mem(0,1);
	   end
	   
	 endcase
	 LAST_clk = _clk;
      end

   specify
      $setuphold(posedge clk, cen, 1.000, 0.500, NOT_cen);
      $setuphold(posedge clk &&& re_flag, oen, 1.000, 0.500, NOT_oen);
      $setuphold(posedge clk &&& re_flag, wen, 1.000, 0.500, NOT_wen);
      $setuphold(posedge clk &&& re_flag, a[0], 1.000, 0.500, NOT_a0);
      $setuphold(posedge clk &&& re_flag, a[1], 1.000, 0.500, NOT_a1);
      $setuphold(posedge clk &&& re_flag, a[2], 1.000, 0.500, NOT_a2);
      $setuphold(posedge clk &&& re_flag, a[3], 1.000, 0.500, NOT_a3);
      $setuphold(posedge clk &&& re_flag, a[4], 1.000, 0.500, NOT_a4);
      $setuphold(posedge clk &&& re_flag, a[5], 1.000, 0.500, NOT_a5);
      $setuphold(posedge clk &&& re_flag, a[6], 1.000, 0.500, NOT_a6);
      $setuphold(posedge clk &&& re_data_flag, d[0], 1.000, 0.500, NOT_d0);
      $setuphold(posedge clk &&& re_data_flag, d[1], 1.000, 0.500, NOT_d1);
      $setuphold(posedge clk &&& re_data_flag, d[2], 1.000, 0.500, NOT_d2);
      $setuphold(posedge clk &&& re_data_flag, d[3], 1.000, 0.500, NOT_d3);
      $setuphold(posedge clk &&& re_data_flag, d[4], 1.000, 0.500, NOT_d4);
      $setuphold(posedge clk &&& re_data_flag, d[5], 1.000, 0.500, NOT_d5);
      $setuphold(posedge clk &&& re_data_flag, d[6], 1.000, 0.500, NOT_d6);
      $setuphold(posedge clk &&& re_data_flag, d[7], 1.000, 0.500, NOT_d7);
      $setuphold(posedge clk &&& re_data_flag, d[8], 1.000, 0.500, NOT_d8);
      $setuphold(posedge clk &&& re_data_flag, d[9], 1.000, 0.500, NOT_d9);
      $setuphold(posedge clk &&& re_data_flag, d[10], 1.000, 0.500, NOT_d10);
      $setuphold(posedge clk &&& re_data_flag, d[11], 1.000, 0.500, NOT_d11);
      $setuphold(posedge clk &&& re_data_flag, d[12], 1.000, 0.500, NOT_d12);
      $setuphold(posedge clk &&& re_data_flag, d[13], 1.000, 0.500, NOT_d13);
      $setuphold(posedge clk &&& re_data_flag, d[14], 1.000, 0.500, NOT_d14);
      $setuphold(posedge clk &&& re_data_flag, d[15], 1.000, 0.500, NOT_d15);
      $setuphold(posedge clk &&& re_data_flag, d[16], 1.000, 0.500, NOT_d16);
      $setuphold(posedge clk &&& re_data_flag, d[17], 1.000, 0.500, NOT_d17);
      $setuphold(posedge clk &&& re_data_flag, d[18], 1.000, 0.500, NOT_d18);
      $setuphold(posedge clk &&& re_data_flag, d[19], 1.000, 0.500, NOT_d19);
      $setuphold(posedge clk &&& re_data_flag, d[20], 1.000, 0.500, NOT_d20);
      $setuphold(posedge clk &&& re_data_flag, d[21], 1.000, 0.500, NOT_d21);
      $setuphold(posedge clk &&& re_data_flag, d[22], 1.000, 0.500, NOT_d22);
      $setuphold(posedge clk &&& re_data_flag, d[23], 1.000, 0.500, NOT_d23);
      $setuphold(posedge clk &&& re_data_flag, d[24], 1.000, 0.500, NOT_d24);
      $setuphold(posedge clk &&& re_data_flag, d[25], 1.000, 0.500, NOT_d25);
      $setuphold(posedge clk &&& re_data_flag, d[26], 1.000, 0.500, NOT_d26);
      $setuphold(posedge clk &&& re_data_flag, d[27], 1.000, 0.500, NOT_d27);
      $setuphold(posedge clk &&& re_data_flag, d[28], 1.000, 0.500, NOT_d28);
      $setuphold(posedge clk &&& re_data_flag, d[29], 1.000, 0.500, NOT_d29);
      $setuphold(posedge clk &&& re_data_flag, d[30], 1.000, 0.500, NOT_d30);
      $setuphold(posedge clk &&& re_data_flag, d[31], 1.000, 0.500, NOT_d31);
      $setuphold(posedge clk &&& re_data_flag, d[32], 1.000, 0.500, NOT_d32);
      $setuphold(posedge clk &&& re_data_flag, d[33], 1.000, 0.500, NOT_d33);
      $setuphold(posedge clk, tis, 1.000, 0.500, NOT_tis);
      $setuphold(posedge clk, tms, 1.000, 0.500, NOT_tms);
      $setuphold(posedge clk, tcen, 1.000, 0.500, NOT_tcen);
      $setuphold(posedge clk &&& re_flag, toen, 1.000, 0.500, NOT_toen);
      $setuphold(posedge clk &&& re_flag, twen, 1.000, 0.500, NOT_twen);
      $setuphold(posedge clk &&& re_flag, ta[0], 1.000, 0.500, NOT_ta0);
      $setuphold(posedge clk &&& re_flag, ta[1], 1.000, 0.500, NOT_ta1);
      $setuphold(posedge clk &&& re_flag, ta[2], 1.000, 0.500, NOT_ta2);
      $setuphold(posedge clk &&& re_flag, ta[3], 1.000, 0.500, NOT_ta3);
      $setuphold(posedge clk &&& re_flag, ta[4], 1.000, 0.500, NOT_ta4);
      $setuphold(posedge clk &&& re_flag, ta[5], 1.000, 0.500, NOT_ta5);
      $setuphold(posedge clk &&& re_flag, ta[6], 1.000, 0.500, NOT_ta6);
      $setuphold(posedge clk &&& re_data_flag, td[0], 1.000, 0.500, NOT_td0);
      $setuphold(posedge clk &&& re_data_flag, td[1], 1.000, 0.500, NOT_td1);
      $setuphold(posedge clk &&& re_data_flag, td[2], 1.000, 0.500, NOT_td2);
      $setuphold(posedge clk &&& re_data_flag, td[3], 1.000, 0.500, NOT_td3);
      $setuphold(posedge clk &&& re_data_flag, td[4], 1.000, 0.500, NOT_td4);
      $setuphold(posedge clk &&& re_data_flag, td[5], 1.000, 0.500, NOT_td5);
      $setuphold(posedge clk &&& re_data_flag, td[6], 1.000, 0.500, NOT_td6);
      $setuphold(posedge clk &&& re_data_flag, td[7], 1.000, 0.500, NOT_td7);
      $setuphold(posedge clk &&& re_data_flag, td[8], 1.000, 0.500, NOT_td8);
      $setuphold(posedge clk &&& re_data_flag, td[9], 1.000, 0.500, NOT_td9);
      $setuphold(posedge clk &&& re_data_flag, td[10], 1.000, 0.500, NOT_td10);
      $setuphold(posedge clk &&& re_data_flag, td[11], 1.000, 0.500, NOT_td11);
      $setuphold(posedge clk &&& re_data_flag, td[12], 1.000, 0.500, NOT_td12);
      $setuphold(posedge clk &&& re_data_flag, td[13], 1.000, 0.500, NOT_td13);
      $setuphold(posedge clk &&& re_data_flag, td[14], 1.000, 0.500, NOT_td14);
      $setuphold(posedge clk &&& re_data_flag, td[15], 1.000, 0.500, NOT_td15);
      $setuphold(posedge clk &&& re_data_flag, td[16], 1.000, 0.500, NOT_td16);
      $setuphold(posedge clk &&& re_data_flag, td[17], 1.000, 0.500, NOT_td17);
      $setuphold(posedge clk &&& re_data_flag, td[18], 1.000, 0.500, NOT_td18);
      $setuphold(posedge clk &&& re_data_flag, td[19], 1.000, 0.500, NOT_td19);
      $setuphold(posedge clk &&& re_data_flag, td[20], 1.000, 0.500, NOT_td20);
      $setuphold(posedge clk &&& re_data_flag, td[21], 1.000, 0.500, NOT_td21);
      $setuphold(posedge clk &&& re_data_flag, td[22], 1.000, 0.500, NOT_td22);
      $setuphold(posedge clk &&& re_data_flag, td[23], 1.000, 0.500, NOT_td23);
      $setuphold(posedge clk &&& re_data_flag, td[24], 1.000, 0.500, NOT_td24);
      $setuphold(posedge clk &&& re_data_flag, td[25], 1.000, 0.500, NOT_td25);
      $setuphold(posedge clk &&& re_data_flag, td[26], 1.000, 0.500, NOT_td26);
      $setuphold(posedge clk &&& re_data_flag, td[27], 1.000, 0.500, NOT_td27);
      $setuphold(posedge clk &&& re_data_flag, td[28], 1.000, 0.500, NOT_td28);
      $setuphold(posedge clk &&& re_data_flag, td[29], 1.000, 0.500, NOT_td29);
      $setuphold(posedge clk &&& re_data_flag, td[30], 1.000, 0.500, NOT_td30);
      $setuphold(posedge clk &&& re_data_flag, td[31], 1.000, 0.500, NOT_td31);
      $setuphold(posedge clk &&& re_data_flag, td[32], 1.000, 0.500, NOT_td32);
      $setuphold(posedge clk &&& re_data_flag, td[33], 1.000, 0.500, NOT_td33);

      $period(posedge clk, 3.000, NOT_clk_PER);
      $width(posedge clk, 1.000, 0, NOT_clk_MINH);
      $width(negedge clk, 1.000, 0, NOT_clk_MINL);

      (clk => tq[0])=(1.000, 1.000);
      (clk => tq[1])=(1.000, 1.000);
      (clk => tq[2])=(1.000, 1.000);
      (clk => tq[3])=(1.000, 1.000);
      (clk => tq[4])=(1.000, 1.000);
      (clk => tq[5])=(1.000, 1.000);
      (clk => tq[6])=(1.000, 1.000);
      (clk => tq[7])=(1.000, 1.000);
      (clk => tq[8])=(1.000, 1.000);
      (clk => tq[9])=(1.000, 1.000);
      (clk => tq[10])=(1.000, 1.000);
      (clk => tq[11])=(1.000, 1.000);
      (clk => tq[12])=(1.000, 1.000);
      (clk => tq[13])=(1.000, 1.000);
      (clk => tq[14])=(1.000, 1.000);
      (clk => tq[15])=(1.000, 1.000);
      (clk => tq[16])=(1.000, 1.000);
      (clk => tq[17])=(1.000, 1.000);
      (clk => tq[18])=(1.000, 1.000);
      (clk => tq[19])=(1.000, 1.000);
      (clk => tq[20])=(1.000, 1.000);
      (clk => tq[21])=(1.000, 1.000);
      (clk => tq[22])=(1.000, 1.000);
      (clk => tq[23])=(1.000, 1.000);
      (clk => tq[24])=(1.000, 1.000);
      (clk => tq[25])=(1.000, 1.000);
      (clk => tq[26])=(1.000, 1.000);
      (clk => tq[27])=(1.000, 1.000);
      (clk => tq[28])=(1.000, 1.000);
      (clk => tq[29])=(1.000, 1.000);
      (clk => tq[30])=(1.000, 1.000);
      (clk => tq[31])=(1.000, 1.000);
      (clk => tq[32])=(1.000, 1.000);
      (clk => tq[33])=(1.000, 1.000);
      if(tms)
         (clk => q[0])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[1])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[2])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[3])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[4])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[5])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[6])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[7])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[8])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[9])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[10])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[11])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[12])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[13])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[14])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[15])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[16])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[17])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[18])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[19])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[20])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[21])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[22])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[23])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[24])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[25])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[26])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[27])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[28])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[29])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[30])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[31])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[32])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(tms)
         (clk => q[33])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[0])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[1])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[2])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[3])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[4])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[5])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[6])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[7])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[8])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[9])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[10])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[11])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[12])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[13])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[14])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[15])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[16])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[17])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[18])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[19])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[20])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[21])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[22])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[23])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[24])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[25])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[26])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[27])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[28])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[29])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[30])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[31])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[32])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      if(~tms)
         (clk => q[33])=(1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (tms => q[0])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[1])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[2])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[3])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[4])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[5])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[6])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[7])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[8])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[9])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[10])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[11])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[12])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[13])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[14])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[15])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[16])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[17])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[18])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[19])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[20])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[21])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[22])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[23])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[24])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[25])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[26])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[27])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[28])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[29])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[30])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[31])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[32])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tms => q[33])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[0])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[1])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[2])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[3])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[4])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[5])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[6])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[7])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[8])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[9])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[10])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[11])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[12])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[13])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[14])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[15])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[16])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[17])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[18])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[19])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[20])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[21])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[22])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[23])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[24])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[25])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[26])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[27])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[28])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[29])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[30])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[31])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[32])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tis => q[33])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[0])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[1])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[2])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[3])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[4])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[5])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[6])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[7])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[8])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[9])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[10])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[11])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[12])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[13])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[14])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[15])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[16])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[17])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[18])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[19])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[20])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[21])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[22])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[23])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[24])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[25])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[26])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[27])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[28])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[29])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[30])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[31])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[32])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (toen => q[33])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[0])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[1])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[2])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[3])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[4])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[5])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[6])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[7])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[8])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[9])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[10])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[11])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[12])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[13])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[14])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[15])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[16])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[17])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[18])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[19])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[20])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[21])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[22])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[23])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[24])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[25])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[26])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[27])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[28])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[29])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[30])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[31])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[32])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (tqoen => tq[33])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (clk => asq[0])=(1.000, 1.000);
      (clk => asq[1])=(1.000, 1.000);
      (clk => asq[2])=(1.000, 1.000);
      (clk => asq[3])=(1.000, 1.000);
      (clk => asq[4])=(1.000, 1.000);
      (clk => asq[5])=(1.000, 1.000);
      (clk => asq[6])=(1.000, 1.000);
      (clk => oensq)=(1.000, 1.000);
      (clk => wensq)=(1.000, 1.000);
      (clk => censq)=(1.000, 1.000);
      (oen => q[0])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[1])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[2])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[3])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[4])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[5])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[6])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[7])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[8])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[9])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[10])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[11])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[12])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[13])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[14])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[15])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[16])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[17])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[18])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[19])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[20])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[21])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[22])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[23])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[24])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[25])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[26])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[27])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[28])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[29])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[30])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[31])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[32])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (oen => q[33])=(1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
   endspecify

endmodule
`endcelldefine
