//
//      CONFIDENTIAL AND PROPRIETARY SOFTWARE/DATA OF ARTISAN COMPONENTS, INC.
//      
//      Copyright (c) 2001 Artisan Components, Inc.  All Rights Reserved.
//      
//      Use of this Software/Data is subject to the terms and conditions of
//      the applicable license agreement between Artisan Components, Inc. and
//      Taiwan Semiconductor Manufacturing Ltd..  In addition, this Software/Data
//      is protected by copyright law and international treaties.
//      
//      The copyright notice(s) in this Software/Data does not indicate actual
//      or intended publication of this Software/Data.
//
//      Verilog model for Synchronous Dual-Port Register File With UTI
//
//      Instance Name:  art_rf2r_32x32
//      Words:          32
//      Word Width:     32
//      Pipeline:       No
//
//      Creation Date:  2001-01-02 20:14:31Z
//      Version: 	99Q1V0
//
//      Verified With: Cadence Verilog-XL
//
//      Modeling Assumptions: This model supports full gate level simulation
//          including proper x-handling and timing check behavior.  Unit
//          delay timing is included in the model. Back-annotation of SDF
//          (v2.1) is supported.  SDF can be created utilyzing the delay
//          calculation views provided with this generator and supported
//          delay calculators.  For netlisting simplicity, buses are
//          not exploded.  All buses are modeled [MSB:LSB].  All ports are
//          padded with Verilog primitives.
//
//      Modeling Limitations: None.
//
//      Known Bugs: None.
//
//      Known Work Arounds: N/A
//
`timescale 1 ns/10 ps
`celldefine

module art_rf2r_32x32 (
   QA,
   OENSQA,
   ASQA,
   TQA,
   QSQA,
   DSOB,
   CENSQB,
   ASQB,
   CLKA,
   CENA,
   OENA,
   AA,
   TISA,
   TMSA,
   TCENA,
   TOENA,
   TQOENA,
   TAA,
   CLKB,
   CENB,
   AB,
   DB,
   TISB,
   TMSB,
   TCENB,
   TAB,
   TDB,
   SMSB,
   DSIB
);
   parameter		   BITS = 32;
   parameter		   word_depth = 32;
   parameter		   addr_width = 5;
   parameter		   wordx = {BITS{1'bx}};
   parameter		   addrx = {addr_width{1'bx}};
	
   output [31:0] QA;
   output OENSQA;
   output [4:0] ASQA;
   output [31:0] TQA;
   output QSQA;
   output DSOB;
   output CENSQB;
   output [4:0] ASQB;
   input CLKA;
   input CENA;
   input OENA;
   input [4:0] AA;
   input TISA;
   input TMSA;
   input TCENA;
   input TOENA;
   input TQOENA;
   input [4:0] TAA;
   input CLKB;
   input CENB;
   input [4:0] AB;
   input [31:0] DB;
   input TISB;
   input TMSB;
   input TCENB;
   input [4:0] TAB;
   input [31:0] TDB;
   input SMSB;
   input DSIB;

   reg [BITS-1:0]	   mem [word_depth-1:0];
   reg                     NOT_CONTA;
   reg                     NOT_CONTB;

   reg			   NOT_CENA;
   reg			   NOT_CENB;
   reg			   NOT_OENA;
   reg			   NOT_AA0;
   reg			   NOT_AA1;
   reg			   NOT_AA2;
   reg			   NOT_AA3;
   reg			   NOT_AA4;
   reg [addr_width-1:0]	   NOT_AA;
   reg			   NOT_AB0;
   reg			   NOT_AB1;
   reg			   NOT_AB2;
   reg			   NOT_AB3;
   reg			   NOT_AB4;
   reg [addr_width-1:0]	   NOT_AB;
   reg			   NOT_DB0;
   reg			   NOT_DB1;
   reg			   NOT_DB2;
   reg			   NOT_DB3;
   reg			   NOT_DB4;
   reg			   NOT_DB5;
   reg			   NOT_DB6;
   reg			   NOT_DB7;
   reg			   NOT_DB8;
   reg			   NOT_DB9;
   reg			   NOT_DB10;
   reg			   NOT_DB11;
   reg			   NOT_DB12;
   reg			   NOT_DB13;
   reg			   NOT_DB14;
   reg			   NOT_DB15;
   reg			   NOT_DB16;
   reg			   NOT_DB17;
   reg			   NOT_DB18;
   reg			   NOT_DB19;
   reg			   NOT_DB20;
   reg			   NOT_DB21;
   reg			   NOT_DB22;
   reg			   NOT_DB23;
   reg			   NOT_DB24;
   reg			   NOT_DB25;
   reg			   NOT_DB26;
   reg			   NOT_DB27;
   reg			   NOT_DB28;
   reg			   NOT_DB29;
   reg			   NOT_DB30;
   reg			   NOT_DB31;
   reg [BITS-1:0]	   NOT_DB;
   reg			   NOT_TISA;
   reg			   NOT_TISB;
   reg			   NOT_TMSA;
   reg			   NOT_TMSB;
   reg			   NOT_TCENA;
   reg			   NOT_TCENB;
   reg			   NOT_TOENA;
   reg			   NOT_DSIB;
   reg			   NOT_SMSB;
   reg			   NOT_TAA0;
   reg			   NOT_TAA1;
   reg			   NOT_TAA2;
   reg			   NOT_TAA3;
   reg			   NOT_TAA4;
   reg [addr_width-1:0]	   NOT_TAA;
   reg			   NOT_TAB0;
   reg			   NOT_TAB1;
   reg			   NOT_TAB2;
   reg			   NOT_TAB3;
   reg			   NOT_TAB4;
   reg [addr_width-1:0]	   NOT_TAB;
   reg			   NOT_TDB0;
   reg			   NOT_TDB1;
   reg			   NOT_TDB2;
   reg			   NOT_TDB3;
   reg			   NOT_TDB4;
   reg			   NOT_TDB5;
   reg			   NOT_TDB6;
   reg			   NOT_TDB7;
   reg			   NOT_TDB8;
   reg			   NOT_TDB9;
   reg			   NOT_TDB10;
   reg			   NOT_TDB11;
   reg			   NOT_TDB12;
   reg			   NOT_TDB13;
   reg			   NOT_TDB14;
   reg			   NOT_TDB15;
   reg			   NOT_TDB16;
   reg			   NOT_TDB17;
   reg			   NOT_TDB18;
   reg			   NOT_TDB19;
   reg			   NOT_TDB20;
   reg			   NOT_TDB21;
   reg			   NOT_TDB22;
   reg			   NOT_TDB23;
   reg			   NOT_TDB24;
   reg			   NOT_TDB25;
   reg			   NOT_TDB26;
   reg			   NOT_TDB27;
   reg			   NOT_TDB28;
   reg			   NOT_TDB29;
   reg			   NOT_TDB30;
   reg			   NOT_TDB31;
   reg [BITS-1:0]	   NOT_TDB;
   reg			   NOT_CLKA_PER;
   reg			   NOT_CLKA_MINH;
   reg			   NOT_CLKA_MINL;
   reg			   NOT_CLKB_PER;
   reg			   NOT_CLKB_MINH;
   reg			   NOT_CLKB_MINL;

   reg			   LAST_NOT_CENA;
   reg			   LAST_NOT_CENB;
   reg			   LAST_NOT_OENA;
   reg			   LAST_NOT_AA0;
   reg			   LAST_NOT_AA1;
   reg			   LAST_NOT_AA2;
   reg			   LAST_NOT_AA3;
   reg			   LAST_NOT_AA4;
   reg [addr_width-1:0]	   LAST_NOT_AA;
   reg			   LAST_NOT_AB0;
   reg			   LAST_NOT_AB1;
   reg			   LAST_NOT_AB2;
   reg			   LAST_NOT_AB3;
   reg			   LAST_NOT_AB4;
   reg [addr_width-1:0]	   LAST_NOT_AB;
   reg			   LAST_NOT_DB0;
   reg			   LAST_NOT_DB1;
   reg			   LAST_NOT_DB2;
   reg			   LAST_NOT_DB3;
   reg			   LAST_NOT_DB4;
   reg			   LAST_NOT_DB5;
   reg			   LAST_NOT_DB6;
   reg			   LAST_NOT_DB7;
   reg			   LAST_NOT_DB8;
   reg			   LAST_NOT_DB9;
   reg			   LAST_NOT_DB10;
   reg			   LAST_NOT_DB11;
   reg			   LAST_NOT_DB12;
   reg			   LAST_NOT_DB13;
   reg			   LAST_NOT_DB14;
   reg			   LAST_NOT_DB15;
   reg			   LAST_NOT_DB16;
   reg			   LAST_NOT_DB17;
   reg			   LAST_NOT_DB18;
   reg			   LAST_NOT_DB19;
   reg			   LAST_NOT_DB20;
   reg			   LAST_NOT_DB21;
   reg			   LAST_NOT_DB22;
   reg			   LAST_NOT_DB23;
   reg			   LAST_NOT_DB24;
   reg			   LAST_NOT_DB25;
   reg			   LAST_NOT_DB26;
   reg			   LAST_NOT_DB27;
   reg			   LAST_NOT_DB28;
   reg			   LAST_NOT_DB29;
   reg			   LAST_NOT_DB30;
   reg			   LAST_NOT_DB31;
   reg [BITS-1:0]	   LAST_NOT_DB;
   reg			   LAST_NOT_TISA;
   reg			   LAST_NOT_TISB;
   reg			   LAST_NOT_TMSA;
   reg			   LAST_NOT_TMSB;
   reg			   LAST_NOT_TCENA;
   reg			   LAST_NOT_TCENB;
   reg			   LAST_NOT_TOENA;
   reg			   LAST_NOT_DSIB;
   reg			   LAST_NOT_SMSB;
   reg			   LAST_NOT_TAA0;
   reg			   LAST_NOT_TAA1;
   reg			   LAST_NOT_TAA2;
   reg			   LAST_NOT_TAA3;
   reg			   LAST_NOT_TAA4;
   reg [addr_width-1:0]	   LAST_NOT_TAA;
   reg			   LAST_NOT_TAB0;
   reg			   LAST_NOT_TAB1;
   reg			   LAST_NOT_TAB2;
   reg			   LAST_NOT_TAB3;
   reg			   LAST_NOT_TAB4;
   reg [addr_width-1:0]	   LAST_NOT_TAB;
   reg			   LAST_NOT_TDB0;
   reg			   LAST_NOT_TDB1;
   reg			   LAST_NOT_TDB2;
   reg			   LAST_NOT_TDB3;
   reg			   LAST_NOT_TDB4;
   reg			   LAST_NOT_TDB5;
   reg			   LAST_NOT_TDB6;
   reg			   LAST_NOT_TDB7;
   reg			   LAST_NOT_TDB8;
   reg			   LAST_NOT_TDB9;
   reg			   LAST_NOT_TDB10;
   reg			   LAST_NOT_TDB11;
   reg			   LAST_NOT_TDB12;
   reg			   LAST_NOT_TDB13;
   reg			   LAST_NOT_TDB14;
   reg			   LAST_NOT_TDB15;
   reg			   LAST_NOT_TDB16;
   reg			   LAST_NOT_TDB17;
   reg			   LAST_NOT_TDB18;
   reg			   LAST_NOT_TDB19;
   reg			   LAST_NOT_TDB20;
   reg			   LAST_NOT_TDB21;
   reg			   LAST_NOT_TDB22;
   reg			   LAST_NOT_TDB23;
   reg			   LAST_NOT_TDB24;
   reg			   LAST_NOT_TDB25;
   reg			   LAST_NOT_TDB26;
   reg			   LAST_NOT_TDB27;
   reg			   LAST_NOT_TDB28;
   reg			   LAST_NOT_TDB29;
   reg			   LAST_NOT_TDB30;
   reg			   LAST_NOT_TDB31;
   reg [BITS-1:0]	   LAST_NOT_TDB;
   reg			   LAST_NOT_CLKA_PER;
   reg			   LAST_NOT_CLKA_MINH;
   reg			   LAST_NOT_CLKA_MINL;
   reg			   LAST_NOT_CLKB_PER;
   reg			   LAST_NOT_CLKB_MINH;
   reg			   LAST_NOT_CLKB_MINL;

   reg                     LAST_NOT_CONTA;
   reg                     LAST_NOT_CONTB;
   wire                    contA_flag;
   wire                    contB_flag;
   wire                    cont_flag;

   wire [BITS-1:0]         _QA;
   wire [addr_width-1:0]   _ASQA;
   wire [addr_width-1:0]   _ASQB;
   wire			   _OENSQA;
   wire			   _CENSQB;
   wire			   _QSQA;
   wire			   _DSOB;
   wire [BITS-1:0]         _TQA;
   wire			   _OENAi;
   wire [addr_width-1:0]   _AA;
   wire [addr_width-1:0]   _AB;
   wire			   _CLKA;
   wire			   _CLKB;
   wire			   _CENA;
   wire			   _CENB;
   wire			   _OENA;
   wire [BITS-1:0]         _DB;
   wire			   _TISA;
   wire			   _TISB;
   wire [addr_width-1:0]   _TAA;
   wire [addr_width-1:0]   _TAB;
   wire			   _TOENAi;
   wire			   _TOENA;
   wire			   _TMSA;
   wire			   _TMSB;
   wire			   _TCENA;
   wire			   _TCENB;
   wire			   _TQOENA;
   wire [BITS-1:0]         _TDB;
   wire                    _DSIB;
   wire                    _SMSB;
   wire                    re_flagA;
   wire                    re_flagB;

   reg			   LATCHED_CENA;
   reg			   LATCHED_CENB;
   reg [addr_width-1:0]	   LATCHED_AA;
   reg [addr_width-1:0]	   LATCHED_AB;
   reg [BITS-1:0]	   LATCHED_DB;
   reg			   LATCHED_TISA;
   reg			   LATCHED_TISB;
   reg			   LATCHED_OENA;
   reg			   LATCHED_TOENA;
   reg			   LATCHED_TMSA;
   reg			   LATCHED_TMSB;
   reg			   LATCHED_TCENA;
   reg			   LATCHED_TCENB;
   reg [addr_width-1:0]	   LATCHED_TAA;
   reg [addr_width-1:0]	   LATCHED_TAB;
   reg [BITS-1:0]	   LATCHED_TDB;
   reg	                   LATCHED_SMSB;
   reg	                   LATCHED_DSIB;

   reg			   CENAi;
   reg			   CENBi;
   reg [addr_width-1:0]	   AAi;
   reg [addr_width-1:0]	   ABi;
   reg [BITS-1:0]	   DBi;
   reg [BITS-1:0]	   QAi;
   reg [BITS-1:0]	   LAST_QAi;

   reg			   CENA_ffin;
   reg			   CENB_ffin;
   reg			   OENA_ffin;
   reg [BITS-1:0]	   QA_ffin;

   reg			   CENA_reg;
   reg			   CENB_reg;
   reg			   OENA_reg;
   reg [addr_width-1:0]	   AA_reg;
   reg [addr_width-1:0]	   AB_reg;
   reg [BITS-1:0]	   DB_reg;
   reg [BITS-1:0]	   LAST_QA_reg;
   reg [BITS-1:0]	   LAST_DB_reg;
   reg 	                   LAST_CENA_reg;
   reg [BITS-1:0]	   QA_reg;

   reg			   LAST_CLKA;
   reg			   LAST_CLKB;

   reg                     valid_cycleA;
   reg                     valid_cycleB;

   task update_Anotifier_buses;
   begin
      NOT_AA = {
               NOT_AA4,
               NOT_AA3,
               NOT_AA2,
               NOT_AA1,
               NOT_AA0};
      NOT_TAA = {
                NOT_TAA4,
                NOT_TAA3,
                NOT_TAA2,
                NOT_TAA1,
                NOT_TAA0};
   end
   endtask

   task update_Bnotifier_buses;
   begin
      NOT_AB = {
               NOT_AB4,
               NOT_AB3,
               NOT_AB2,
               NOT_AB1,
               NOT_AB0};
      NOT_DB = {
               NOT_DB31,
               NOT_DB30,
               NOT_DB29,
               NOT_DB28,
               NOT_DB27,
               NOT_DB26,
               NOT_DB25,
               NOT_DB24,
               NOT_DB23,
               NOT_DB22,
               NOT_DB21,
               NOT_DB20,
               NOT_DB19,
               NOT_DB18,
               NOT_DB17,
               NOT_DB16,
               NOT_DB15,
               NOT_DB14,
               NOT_DB13,
               NOT_DB12,
               NOT_DB11,
               NOT_DB10,
               NOT_DB9,
               NOT_DB8,
               NOT_DB7,
               NOT_DB6,
               NOT_DB5,
               NOT_DB4,
               NOT_DB3,
               NOT_DB2,
               NOT_DB1,
               NOT_DB0};
      NOT_TAB = {
                NOT_TAB4,
                NOT_TAB3,
                NOT_TAB2,
                NOT_TAB1,
                NOT_TAB0};
      NOT_TDB = {
                NOT_TDB31,
                NOT_TDB30,
                NOT_TDB29,
                NOT_TDB28,
                NOT_TDB27,
                NOT_TDB26,
                NOT_TDB25,
                NOT_TDB24,
                NOT_TDB23,
                NOT_TDB22,
                NOT_TDB21,
                NOT_TDB20,
                NOT_TDB19,
                NOT_TDB18,
                NOT_TDB17,
                NOT_TDB16,
                NOT_TDB15,
                NOT_TDB14,
                NOT_TDB13,
                NOT_TDB12,
                NOT_TDB11,
                NOT_TDB10,
                NOT_TDB9,
                NOT_TDB8,
                NOT_TDB7,
                NOT_TDB6,
                NOT_TDB5,
                NOT_TDB4,
                NOT_TDB3,
                NOT_TDB2,
                NOT_TDB1,
                NOT_TDB0};
   end
   endtask

   task mem_cycleA;
   begin
      valid_cycleA = 1'bx;
      update_CENAflop(0);
      casez({CENAi})
	1'b0: begin
	   valid_cycleA = 1;
	   update_Aflops(0);
	   read_memA(1,0);
	end
	1'b1: ;
	1'bx: begin
	   valid_cycleA = 1;
	   update_Aflops(1);
	   read_memA(0,1);
	end
      endcase
   end
   endtask
      
   task mem_cycleB;
   begin
      valid_cycleB = 1'bx;
      update_CENBflop(0);
      casez({CENBi,LATCHED_TMSB})
	2'b0?: begin
	   valid_cycleB = 0;
	   update_Bflops(0);
	   write_mem(ABi,DBi);
	end
	2'b11: begin
	   update_Bflops(0);
	end
        2'b1x: begin
	   update_Bflops(1);
        end
	2'b10: ;
        2'bx1: begin
	   valid_cycleB = 0;
	   update_Bflops(0);
	   write_mem_x(ABi);
	end
        2'bx0: begin
	   valid_cycleB = 0;
	   update_Bflops(1);
	   write_mem_x(ABi);
	end
        2'bxx: begin
	   valid_cycleB = 0;
	   update_Bflops(1);
	   write_mem_x(ABi);
	end
      endcase
   end
   endtask
      
   task contentionA;
   begin
      casez(valid_cycleB)
	1'bx: ;
	1'b0:begin
	   read_memA(0,1);
	end
	1'b1: ;
      endcase
   end
   endtask

   task contentionB;
   begin
      casez(valid_cycleA)
	1'bx: ;
	1'b1:begin
	   read_memA(0,1);
	end
	1'b0: ;
      endcase
   end
   endtask

   task update_Alast_notifiers;
   begin
      LAST_NOT_AA = NOT_AA;
      LAST_NOT_OENA = NOT_OENA;
      LAST_NOT_CENA = NOT_CENA;
      LAST_NOT_TISA = NOT_TISA;
      LAST_NOT_TOENA = NOT_TOENA;
      LAST_NOT_TMSA = NOT_TMSA;
      LAST_NOT_TCENA = NOT_TCENA;
      LAST_NOT_TAA = NOT_TAA;
      LAST_NOT_CLKA_PER = NOT_CLKA_PER;
      LAST_NOT_CLKA_MINH = NOT_CLKA_MINH;
      LAST_NOT_CLKA_MINL = NOT_CLKA_MINL;
      LAST_NOT_CONTA = NOT_CONTA;
   end
   endtask

   task update_Blast_notifiers;
   begin
      LAST_NOT_AB = NOT_AB;
      LAST_NOT_DB = NOT_DB;
      LAST_NOT_CENB = NOT_CENB;
      LAST_NOT_TISB = NOT_TISB;
      LAST_NOT_TMSB = NOT_TMSB;
      LAST_NOT_TCENB = NOT_TCENB;
      LAST_NOT_TAB = NOT_TAB;
      LAST_NOT_TDB = NOT_TDB;
      LAST_NOT_SMSB = NOT_SMSB;
      LAST_NOT_DSIB = NOT_DSIB;
      LAST_NOT_CLKB_PER = NOT_CLKB_PER;
      LAST_NOT_CLKB_MINH = NOT_CLKB_MINH;
      LAST_NOT_CLKB_MINL = NOT_CLKB_MINL;
      LAST_NOT_CONTB = NOT_CONTB;
   end
   endtask

   task latch_Ainputs;
   begin
      LATCHED_AA = _AA ;
      LATCHED_CENA = _CENA ;
      LATCHED_TISA = _TISA ;
      LATCHED_OENA = _OENA ;
      LATCHED_TOENA = _TOENA ;
      LATCHED_TMSA = _TMSA ;
      LATCHED_TCENA = _TCENA ;
      LATCHED_TAA = _TAA ;
      LAST_QA_reg = QA_reg;
      LAST_CENA_reg = CENA_reg;
      LAST_QAi = QAi;
   end
   endtask

   task latch_Binputs;
   begin
      LATCHED_AB = _AB ;
      LATCHED_DB = _DB ;
      LATCHED_CENB = _CENB ;
      LATCHED_TISB = _TISB ;
      LATCHED_TMSB = _TMSB ;
      LATCHED_TCENB = _TCENB ;
      LATCHED_TAB = _TAB ;
      LATCHED_TDB = _TDB ;
      LATCHED_SMSB = _SMSB ;
      LATCHED_DSIB = _DSIB ;
      LAST_DB_reg = DB_reg;
   end
   endtask

   task update_Alogic;
      integer n;
   begin
      CENAi = !(LATCHED_TMSA || !(LATCHED_TISA ? LATCHED_TCENA : LATCHED_CENA));
      AAi = (LATCHED_TISA ? LATCHED_TAA : LATCHED_AA) ;
      CENA_ffin = (LATCHED_TISA ? LATCHED_TCENA : LATCHED_CENA);
      OENA_ffin = (LATCHED_TISA ? LATCHED_TOENA : LATCHED_OENA);
      QA_ffin[0] = (LATCHED_TMSA ? LAST_CENA_reg : LAST_QAi[0]);
      for (n=1; n<BITS; n=n+1) begin
         QA_ffin[n] = (LATCHED_TMSA ? LAST_QA_reg[n-1] : LAST_QAi[n]);
      end
   end
   endtask

   task update_Blogic;
      integer n;
   begin
      CENBi = (LATCHED_TMSB || (LATCHED_TISB ? LATCHED_TCENB : LATCHED_CENB));
      ABi = (LATCHED_TISB ? LATCHED_TAB : LATCHED_AB) ;
      DBi[0] = (LATCHED_SMSB ? LATCHED_DSIB : (LATCHED_TISB ? LATCHED_TDB[0] : LATCHED_DB[0]));
      for (n=1; n<BITS; n=n+1) begin
         DBi[n] = (LATCHED_SMSB ? LAST_DB_reg[n-1] : (LATCHED_TISB ? LATCHED_TDB[n] : LATCHED_DB[n]));
      end
      CENB_ffin = (LATCHED_TISB ? LATCHED_TCENB : LATCHED_CENB);
   end
   endtask


   task update_Aflops;
      input xflag;
   begin
      AA_reg = xflag ? addrx : AAi;
      QA_reg = xflag ? wordx : QA_ffin ;
      OENA_reg = xflag ? 1'bx : OENA_ffin ;
   end
   endtask

   task update_CENAflop;
      input xflag;
   begin
      CENA_reg = xflag ? 1'bx : CENA_ffin ;
   end
   endtask

   task update_Bflops;
      input xflag;
   begin
      AB_reg = xflag ? addrx : ABi;
      DB_reg = xflag ? wordx : DBi ;
   end
   endtask

   task update_CENBflop;
      input xflag;
   begin
      CENB_reg = xflag ? 1'bx : CENB_ffin ;
   end
   endtask

   task x_Ainputs;
      integer n;
   begin
      for (n=0; n<addr_width; n=n+1)
	 begin
	    LATCHED_AA[n] = (NOT_AA[n]!==LAST_NOT_AA[n]) ? 1'bx : LATCHED_AA[n] ;
	    LATCHED_TAA[n] = (NOT_TAA[n]!==LAST_NOT_TAA[n]) ? 1'bx : LATCHED_TAA[n] ;
	 end
      LATCHED_OENA = (NOT_OENA!==LAST_NOT_OENA) ? 1'bx : LATCHED_OENA ;
      LATCHED_CENA = (NOT_CENA!==LAST_NOT_CENA) ? 1'bx : LATCHED_CENA ;
      LATCHED_TISA = (NOT_TISA!==LAST_NOT_TISA) ? 1'bx : LATCHED_TISA ;
      LATCHED_TOENA = (NOT_TOENA!==LAST_NOT_TOENA) ? 1'bx : LATCHED_TOENA ;
      LATCHED_TMSA = (NOT_TMSA!==LAST_NOT_TMSA) ? 1'bx : LATCHED_TMSA ;
      LATCHED_TCENA = (NOT_TCENA!==LAST_NOT_TCENA) ? 1'bx : LATCHED_TCENA ;
   end
   endtask

   task x_Binputs;
      integer n;
   begin
      for (n=0; n<addr_width; n=n+1)
	 begin
	    LATCHED_AB[n] = (NOT_AB[n]!==LAST_NOT_AB[n]) ? 1'bx : LATCHED_AB[n] ;
	    LATCHED_TAB[n] = (NOT_TAB[n]!==LAST_NOT_TAB[n]) ? 1'bx : LATCHED_TAB[n] ;
	 end
      for (n=0; n<BITS; n=n+1)
	 begin
	    LATCHED_DB[n] = (NOT_DB[n]!==LAST_NOT_DB[n]) ? 1'bx : LATCHED_DB[n] ;
	    LATCHED_TDB[n] = (NOT_TDB[n]!==LAST_NOT_TDB[n]) ? 1'bx : LATCHED_TDB[n] ;
	 end
      LATCHED_CENB = (NOT_CENB!==LAST_NOT_CENB) ? 1'bx : LATCHED_CENB ;
      LATCHED_TISB = (NOT_TISB!==LAST_NOT_TISB) ? 1'bx : LATCHED_TISB ;
      LATCHED_TMSB = (NOT_TMSB!==LAST_NOT_TMSB) ? 1'bx : LATCHED_TMSB ;
      LATCHED_TCENB = (NOT_TCENB!==LAST_NOT_TCENB) ? 1'bx : LATCHED_TCENB ;
      LATCHED_DSIB = (NOT_DSIB!==LAST_NOT_DSIB) ? 1'bx : LATCHED_DSIB ;
      LATCHED_SMSB = (NOT_SMSB!==LAST_NOT_SMSB) ? 1'bx : LATCHED_SMSB ;
   end
   endtask

   task read_memA;
      input r_wb;
      input xflag;
   begin
      if (r_wb)
	 begin
	    if (valid_address(AAi))
	       begin
	          QAi=mem[AAi];
	       end
	    else
	       begin
		  QAi=wordx;
	       end
	 end
      else
	 begin
	    if (xflag)
	       begin
		  QAi=wordx;
	       end
	    else
	       begin
	          QAi=QAi;
	       end
	 end
   end
   endtask

   task write_mem;
      input [addr_width-1:0] a;
      input [BITS-1:0] d;
   begin
      casez({valid_address(a)})
	1'b0: x_mem;
	1'b1: mem[a]=d;
      endcase
   end
   endtask

   task write_mem_x;
      input [addr_width-1:0] a;
   begin
      casez({valid_address(a)})
	1'b0: x_mem;
	1'b1: mem[a]=wordx;
      endcase
   end
   endtask

   task x_mem;
      integer n;
   begin
      for (n=0; n<word_depth; n=n+1)
	 mem[n]=wordx;
   end
   endtask

   function valid_address;
      input [addr_width-1:0] a;
   begin
      valid_address = (^(a) !== 1'bx);
   end
   endfunction

   buf (ASQA[0], _ASQA[0]);
   buf (ASQA[1], _ASQA[1]);
   buf (ASQA[2], _ASQA[2]);
   buf (ASQA[3], _ASQA[3]);
   buf (ASQA[4], _ASQA[4]);
   buf (OENSQA, _OENSQA);
   bufif0 (QA[0], _QA[0], _OENAi);
   bufif0 (QA[1], _QA[1], _OENAi);
   bufif0 (QA[2], _QA[2], _OENAi);
   bufif0 (QA[3], _QA[3], _OENAi);
   bufif0 (QA[4], _QA[4], _OENAi);
   bufif0 (QA[5], _QA[5], _OENAi);
   bufif0 (QA[6], _QA[6], _OENAi);
   bufif0 (QA[7], _QA[7], _OENAi);
   bufif0 (QA[8], _QA[8], _OENAi);
   bufif0 (QA[9], _QA[9], _OENAi);
   bufif0 (QA[10], _QA[10], _OENAi);
   bufif0 (QA[11], _QA[11], _OENAi);
   bufif0 (QA[12], _QA[12], _OENAi);
   bufif0 (QA[13], _QA[13], _OENAi);
   bufif0 (QA[14], _QA[14], _OENAi);
   bufif0 (QA[15], _QA[15], _OENAi);
   bufif0 (QA[16], _QA[16], _OENAi);
   bufif0 (QA[17], _QA[17], _OENAi);
   bufif0 (QA[18], _QA[18], _OENAi);
   bufif0 (QA[19], _QA[19], _OENAi);
   bufif0 (QA[20], _QA[20], _OENAi);
   bufif0 (QA[21], _QA[21], _OENAi);
   bufif0 (QA[22], _QA[22], _OENAi);
   bufif0 (QA[23], _QA[23], _OENAi);
   bufif0 (QA[24], _QA[24], _OENAi);
   bufif0 (QA[25], _QA[25], _OENAi);
   bufif0 (QA[26], _QA[26], _OENAi);
   bufif0 (QA[27], _QA[27], _OENAi);
   bufif0 (QA[28], _QA[28], _OENAi);
   bufif0 (QA[29], _QA[29], _OENAi);
   bufif0 (QA[30], _QA[30], _OENAi);
   bufif0 (QA[31], _QA[31], _OENAi);
   bufif1 (TQA[0], _TQA[0],_TOENAi);
   bufif1 (TQA[1], _TQA[1],_TOENAi);
   bufif1 (TQA[2], _TQA[2],_TOENAi);
   bufif1 (TQA[3], _TQA[3],_TOENAi);
   bufif1 (TQA[4], _TQA[4],_TOENAi);
   bufif1 (TQA[5], _TQA[5],_TOENAi);
   bufif1 (TQA[6], _TQA[6],_TOENAi);
   bufif1 (TQA[7], _TQA[7],_TOENAi);
   bufif1 (TQA[8], _TQA[8],_TOENAi);
   bufif1 (TQA[9], _TQA[9],_TOENAi);
   bufif1 (TQA[10], _TQA[10],_TOENAi);
   bufif1 (TQA[11], _TQA[11],_TOENAi);
   bufif1 (TQA[12], _TQA[12],_TOENAi);
   bufif1 (TQA[13], _TQA[13],_TOENAi);
   bufif1 (TQA[14], _TQA[14],_TOENAi);
   bufif1 (TQA[15], _TQA[15],_TOENAi);
   bufif1 (TQA[16], _TQA[16],_TOENAi);
   bufif1 (TQA[17], _TQA[17],_TOENAi);
   bufif1 (TQA[18], _TQA[18],_TOENAi);
   bufif1 (TQA[19], _TQA[19],_TOENAi);
   bufif1 (TQA[20], _TQA[20],_TOENAi);
   bufif1 (TQA[21], _TQA[21],_TOENAi);
   bufif1 (TQA[22], _TQA[22],_TOENAi);
   bufif1 (TQA[23], _TQA[23],_TOENAi);
   bufif1 (TQA[24], _TQA[24],_TOENAi);
   bufif1 (TQA[25], _TQA[25],_TOENAi);
   bufif1 (TQA[26], _TQA[26],_TOENAi);
   bufif1 (TQA[27], _TQA[27],_TOENAi);
   bufif1 (TQA[28], _TQA[28],_TOENAi);
   bufif1 (TQA[29], _TQA[29],_TOENAi);
   bufif1 (TQA[30], _TQA[30],_TOENAi);
   bufif1 (TQA[31], _TQA[31],_TOENAi);
   buf (_AA[0], AA[0]);
   buf (_AA[1], AA[1]);
   buf (_AA[2], AA[2]);
   buf (_AA[3], AA[3]);
   buf (_AA[4], AA[4]);
   buf (_CLKA, CLKA);
   buf (_OENA, OENA);
   buf (_CENA, CENA);
   buf (_TISA, TISA);
   buf (_TOENA, TOENA);
   buf (_TMSA, TMSA);
   buf (_TCENA, TCENA);
   buf (_TQOENA, TQOENA);
   buf (_TAA[0], TAA[0]);
   buf (_TAA[1], TAA[1]);
   buf (_TAA[2], TAA[2]);
   buf (_TAA[3], TAA[3]);
   buf (_TAA[4], TAA[4]);

   buf (ASQB[0], _ASQB[0]);
   buf (ASQB[1], _ASQB[1]);
   buf (ASQB[2], _ASQB[2]);
   buf (ASQB[3], _ASQB[3]);
   buf (ASQB[4], _ASQB[4]);
   buf (CENSQB, _CENSQB);
   buf (QSQA, _QSQA);
   buf (DSOB, _DSOB);
   buf (_DB[0], DB[0]);
   buf (_DB[1], DB[1]);
   buf (_DB[2], DB[2]);
   buf (_DB[3], DB[3]);
   buf (_DB[4], DB[4]);
   buf (_DB[5], DB[5]);
   buf (_DB[6], DB[6]);
   buf (_DB[7], DB[7]);
   buf (_DB[8], DB[8]);
   buf (_DB[9], DB[9]);
   buf (_DB[10], DB[10]);
   buf (_DB[11], DB[11]);
   buf (_DB[12], DB[12]);
   buf (_DB[13], DB[13]);
   buf (_DB[14], DB[14]);
   buf (_DB[15], DB[15]);
   buf (_DB[16], DB[16]);
   buf (_DB[17], DB[17]);
   buf (_DB[18], DB[18]);
   buf (_DB[19], DB[19]);
   buf (_DB[20], DB[20]);
   buf (_DB[21], DB[21]);
   buf (_DB[22], DB[22]);
   buf (_DB[23], DB[23]);
   buf (_DB[24], DB[24]);
   buf (_DB[25], DB[25]);
   buf (_DB[26], DB[26]);
   buf (_DB[27], DB[27]);
   buf (_DB[28], DB[28]);
   buf (_DB[29], DB[29]);
   buf (_DB[30], DB[30]);
   buf (_DB[31], DB[31]);
   buf (_AB[0], AB[0]);
   buf (_AB[1], AB[1]);
   buf (_AB[2], AB[2]);
   buf (_AB[3], AB[3]);
   buf (_AB[4], AB[4]);
   buf (_CLKB, CLKB);
   buf (_CENB, CENB);
   buf (_TISB, TISB);
   buf (_TMSB, TMSB);
   buf (_SMSB, SMSB);
   buf (_DSIB, DSIB);
   buf (_TCENB, TCENB);
   buf (_TAB[0], TAB[0]);
   buf (_TAB[1], TAB[1]);
   buf (_TAB[2], TAB[2]);
   buf (_TAB[3], TAB[3]);
   buf (_TAB[4], TAB[4]);
   buf (_TDB[0], TDB[0]);
   buf (_TDB[1], TDB[1]);
   buf (_TDB[2], TDB[2]);
   buf (_TDB[3], TDB[3]);
   buf (_TDB[4], TDB[4]);
   buf (_TDB[5], TDB[5]);
   buf (_TDB[6], TDB[6]);
   buf (_TDB[7], TDB[7]);
   buf (_TDB[8], TDB[8]);
   buf (_TDB[9], TDB[9]);
   buf (_TDB[10], TDB[10]);
   buf (_TDB[11], TDB[11]);
   buf (_TDB[12], TDB[12]);
   buf (_TDB[13], TDB[13]);
   buf (_TDB[14], TDB[14]);
   buf (_TDB[15], TDB[15]);
   buf (_TDB[16], TDB[16]);
   buf (_TDB[17], TDB[17]);
   buf (_TDB[18], TDB[18]);
   buf (_TDB[19], TDB[19]);
   buf (_TDB[20], TDB[20]);
   buf (_TDB[21], TDB[21]);
   buf (_TDB[22], TDB[22]);
   buf (_TDB[23], TDB[23]);
   buf (_TDB[24], TDB[24]);
   buf (_TDB[25], TDB[25]);
   buf (_TDB[26], TDB[26]);
   buf (_TDB[27], TDB[27]);
   buf (_TDB[28], TDB[28]);
   buf (_TDB[29], TDB[29]);
   buf (_TDB[30], TDB[30]);
   buf (_TDB[31], TDB[31]);

   assign _OENAi = (_TMSA || ( _TISA ? _TOENA : _OENA) );
   assign _TOENAi = ~_TQOENA ;
   assign _OENSQA = OENA_reg;
   assign _QSQA = QA_reg[BITS-1];
   assign _ASQA = AA_reg;
   assign _TQA = QA_reg;
   assign re_flagA = (_TMSA || !( _TISA ? _TCENA : _CENA));
   assign _QA = QAi;

   assign _DSOB = DB_reg[BITS-1];
   assign _CENSQB = CENB_reg;
   assign _ASQB = AB_reg;
   assign re_flagB = (_TMSB || !( _TISB ? _TCENB : _CENB));

   assign contA_flag = 
      ( ( _TISA ? _TAA : _AA ) === ABi ) && 
      ( (!(_TMSA) && (_TISA ? _TCENA : _CENA)) !== 1'b1) &&
      (CENBi !== 1'b1);
   
   assign contB_flag = 
      ( ( _TISB ? _TAB : _AB ) === AAi ) && 
      ( ((_TMSB) || (_TISB ? _TCENB : _CENB)) !== 1'b1) &&
      (CENAi !== 1'b1);

   assign cont_flag = 
      ( ( _TISB ? _TAB : _AB ) === ( _TISA ? _TAA : _AA ) ) && 
      ( ((_TMSB) || (_TISB ? _TCENB : _CENB)) !== 1'b1) &&
      ( (!(_TMSA) && (_TISA ? _TCENA : _CENA)) !== 1'b1);

   always @(
	    NOT_AA0 or
	    NOT_AA1 or
	    NOT_AA2 or
	    NOT_AA3 or
	    NOT_AA4 or
	    NOT_OENA or
	    NOT_CENA or
	    NOT_TISA or
	    NOT_TOENA or
	    NOT_TMSA or
	    NOT_TCENA or
	    NOT_TAA0 or
	    NOT_TAA1 or
	    NOT_TAA2 or
	    NOT_TAA3 or
	    NOT_TAA4 or
            NOT_CONTA or
	    NOT_CLKA_PER or
	    NOT_CLKA_MINH or
	    NOT_CLKA_MINL
	    )
      begin
	 if ((NOT_CLKA_PER!==LAST_NOT_CLKA_PER) ||
	     (NOT_CLKA_MINH!==LAST_NOT_CLKA_MINH) ||
	     (NOT_CLKA_MINL!==LAST_NOT_CLKA_MINL))
	    begin
               update_CENAflop(1);
	       if (CENAi !== 1'b1)
		  begin
		     update_Aflops(1);
		     read_memA(0,1);
		  end
	    end
	 else
	    begin
	       update_Anotifier_buses;
	       x_Ainputs;
	       update_Alogic;
               if (NOT_CONTA!==LAST_NOT_CONTA)
                  begin
		     contentionA;
                  end
               else
                  begin
                     mem_cycleA;
                  end
	    end
	 update_Alast_notifiers;
      end

   always @(
	    NOT_AB0 or
	    NOT_AB1 or
	    NOT_AB2 or
	    NOT_AB3 or
	    NOT_AB4 or
	    NOT_DB0 or
	    NOT_DB1 or
	    NOT_DB2 or
	    NOT_DB3 or
	    NOT_DB4 or
	    NOT_DB5 or
	    NOT_DB6 or
	    NOT_DB7 or
	    NOT_DB8 or
	    NOT_DB9 or
	    NOT_DB10 or
	    NOT_DB11 or
	    NOT_DB12 or
	    NOT_DB13 or
	    NOT_DB14 or
	    NOT_DB15 or
	    NOT_DB16 or
	    NOT_DB17 or
	    NOT_DB18 or
	    NOT_DB19 or
	    NOT_DB20 or
	    NOT_DB21 or
	    NOT_DB22 or
	    NOT_DB23 or
	    NOT_DB24 or
	    NOT_DB25 or
	    NOT_DB26 or
	    NOT_DB27 or
	    NOT_DB28 or
	    NOT_DB29 or
	    NOT_DB30 or
	    NOT_DB31 or
	    NOT_CENB or
	    NOT_TISB or
	    NOT_TMSB or
	    NOT_SMSB or
	    NOT_DSIB or
	    NOT_TCENB or
	    NOT_TAB0 or
	    NOT_TAB1 or
	    NOT_TAB2 or
	    NOT_TAB3 or
	    NOT_TAB4 or
	    NOT_TDB0 or
	    NOT_TDB1 or
	    NOT_TDB2 or
	    NOT_TDB3 or
	    NOT_TDB4 or
	    NOT_TDB5 or
	    NOT_TDB6 or
	    NOT_TDB7 or
	    NOT_TDB8 or
	    NOT_TDB9 or
	    NOT_TDB10 or
	    NOT_TDB11 or
	    NOT_TDB12 or
	    NOT_TDB13 or
	    NOT_TDB14 or
	    NOT_TDB15 or
	    NOT_TDB16 or
	    NOT_TDB17 or
	    NOT_TDB18 or
	    NOT_TDB19 or
	    NOT_TDB20 or
	    NOT_TDB21 or
	    NOT_TDB22 or
	    NOT_TDB23 or
	    NOT_TDB24 or
	    NOT_TDB25 or
	    NOT_TDB26 or
	    NOT_TDB27 or
	    NOT_TDB28 or
	    NOT_TDB29 or
	    NOT_TDB30 or
	    NOT_TDB31 or
            NOT_CONTB or
	    NOT_CLKB_PER or
	    NOT_CLKB_MINH or
	    NOT_CLKB_MINL
	    )
      begin
	 if ((NOT_CLKB_PER!==LAST_NOT_CLKB_PER) ||
	     (NOT_CLKB_MINH!==LAST_NOT_CLKB_MINH) ||
	     (NOT_CLKB_MINL!==LAST_NOT_CLKB_MINL))
	    begin
               update_CENBflop(1);
	       if (CENBi !== 1'b1)
		  begin
		     update_Bflops(1);
		     x_mem;
		     read_memA(0,1);
		  end
	    end
	 else
	    begin
	       update_Bnotifier_buses;
	       x_Binputs;
	       update_Blogic;
               if (NOT_CONTB!==LAST_NOT_CONTB)
                  begin
		     contentionB;
                  end
               else
                  begin
                     mem_cycleB;
                  end
	    end
	 update_Blast_notifiers;
      end

   always @( _CLKA )
      begin
         casez({LAST_CLKA,_CLKA})
	   2'b01: begin
	      latch_Ainputs;
	      update_Alogic;
	      mem_cycleA;
	   end

	   2'b10,
	   2'bx?,
	   2'b00,
	   2'b11: ;

	   2'b?x: begin
	      update_Aflops(1);
              update_CENAflop(1);
              read_memA(0,1);
	   end
	   
	 endcase
	 LAST_CLKA = _CLKA;
      end

   always @( _CLKB )
      begin
         casez({LAST_CLKB,_CLKB})
	   2'b01: begin
	      latch_Binputs;
	      update_Blogic;
	      mem_cycleB;
	   end

	   2'b10,
	   2'bx?,
	   2'b00,
	   2'b11: ;

	   2'b?x: begin
	      update_Bflops(1);
              update_CENBflop(1);
	      x_mem;
	   end
	   
	 endcase
	 LAST_CLKB = _CLKB;
      end

   specify
      $setuphold(posedge CLKB &&& re_flagB, DB[0], 1.000, 0.500, NOT_DB0);
      $setuphold(posedge CLKB &&& re_flagB, DB[1], 1.000, 0.500, NOT_DB1);
      $setuphold(posedge CLKB &&& re_flagB, DB[2], 1.000, 0.500, NOT_DB2);
      $setuphold(posedge CLKB &&& re_flagB, DB[3], 1.000, 0.500, NOT_DB3);
      $setuphold(posedge CLKB &&& re_flagB, DB[4], 1.000, 0.500, NOT_DB4);
      $setuphold(posedge CLKB &&& re_flagB, DB[5], 1.000, 0.500, NOT_DB5);
      $setuphold(posedge CLKB &&& re_flagB, DB[6], 1.000, 0.500, NOT_DB6);
      $setuphold(posedge CLKB &&& re_flagB, DB[7], 1.000, 0.500, NOT_DB7);
      $setuphold(posedge CLKB &&& re_flagB, DB[8], 1.000, 0.500, NOT_DB8);
      $setuphold(posedge CLKB &&& re_flagB, DB[9], 1.000, 0.500, NOT_DB9);
      $setuphold(posedge CLKB &&& re_flagB, DB[10], 1.000, 0.500, NOT_DB10);
      $setuphold(posedge CLKB &&& re_flagB, DB[11], 1.000, 0.500, NOT_DB11);
      $setuphold(posedge CLKB &&& re_flagB, DB[12], 1.000, 0.500, NOT_DB12);
      $setuphold(posedge CLKB &&& re_flagB, DB[13], 1.000, 0.500, NOT_DB13);
      $setuphold(posedge CLKB &&& re_flagB, DB[14], 1.000, 0.500, NOT_DB14);
      $setuphold(posedge CLKB &&& re_flagB, DB[15], 1.000, 0.500, NOT_DB15);
      $setuphold(posedge CLKB &&& re_flagB, DB[16], 1.000, 0.500, NOT_DB16);
      $setuphold(posedge CLKB &&& re_flagB, DB[17], 1.000, 0.500, NOT_DB17);
      $setuphold(posedge CLKB &&& re_flagB, DB[18], 1.000, 0.500, NOT_DB18);
      $setuphold(posedge CLKB &&& re_flagB, DB[19], 1.000, 0.500, NOT_DB19);
      $setuphold(posedge CLKB &&& re_flagB, DB[20], 1.000, 0.500, NOT_DB20);
      $setuphold(posedge CLKB &&& re_flagB, DB[21], 1.000, 0.500, NOT_DB21);
      $setuphold(posedge CLKB &&& re_flagB, DB[22], 1.000, 0.500, NOT_DB22);
      $setuphold(posedge CLKB &&& re_flagB, DB[23], 1.000, 0.500, NOT_DB23);
      $setuphold(posedge CLKB &&& re_flagB, DB[24], 1.000, 0.500, NOT_DB24);
      $setuphold(posedge CLKB &&& re_flagB, DB[25], 1.000, 0.500, NOT_DB25);
      $setuphold(posedge CLKB &&& re_flagB, DB[26], 1.000, 0.500, NOT_DB26);
      $setuphold(posedge CLKB &&& re_flagB, DB[27], 1.000, 0.500, NOT_DB27);
      $setuphold(posedge CLKB &&& re_flagB, DB[28], 1.000, 0.500, NOT_DB28);
      $setuphold(posedge CLKB &&& re_flagB, DB[29], 1.000, 0.500, NOT_DB29);
      $setuphold(posedge CLKB &&& re_flagB, DB[30], 1.000, 0.500, NOT_DB30);
      $setuphold(posedge CLKB &&& re_flagB, DB[31], 1.000, 0.500, NOT_DB31);
      $setuphold(posedge CLKB &&& re_flagB, TDB[0], 1.000, 0.500, NOT_TDB0);
      $setuphold(posedge CLKB &&& re_flagB, TDB[1], 1.000, 0.500, NOT_TDB1);
      $setuphold(posedge CLKB &&& re_flagB, TDB[2], 1.000, 0.500, NOT_TDB2);
      $setuphold(posedge CLKB &&& re_flagB, TDB[3], 1.000, 0.500, NOT_TDB3);
      $setuphold(posedge CLKB &&& re_flagB, TDB[4], 1.000, 0.500, NOT_TDB4);
      $setuphold(posedge CLKB &&& re_flagB, TDB[5], 1.000, 0.500, NOT_TDB5);
      $setuphold(posedge CLKB &&& re_flagB, TDB[6], 1.000, 0.500, NOT_TDB6);
      $setuphold(posedge CLKB &&& re_flagB, TDB[7], 1.000, 0.500, NOT_TDB7);
      $setuphold(posedge CLKB &&& re_flagB, TDB[8], 1.000, 0.500, NOT_TDB8);
      $setuphold(posedge CLKB &&& re_flagB, TDB[9], 1.000, 0.500, NOT_TDB9);
      $setuphold(posedge CLKB &&& re_flagB, TDB[10], 1.000, 0.500, NOT_TDB10);
      $setuphold(posedge CLKB &&& re_flagB, TDB[11], 1.000, 0.500, NOT_TDB11);
      $setuphold(posedge CLKB &&& re_flagB, TDB[12], 1.000, 0.500, NOT_TDB12);
      $setuphold(posedge CLKB &&& re_flagB, TDB[13], 1.000, 0.500, NOT_TDB13);
      $setuphold(posedge CLKB &&& re_flagB, TDB[14], 1.000, 0.500, NOT_TDB14);
      $setuphold(posedge CLKB &&& re_flagB, TDB[15], 1.000, 0.500, NOT_TDB15);
      $setuphold(posedge CLKB &&& re_flagB, TDB[16], 1.000, 0.500, NOT_TDB16);
      $setuphold(posedge CLKB &&& re_flagB, TDB[17], 1.000, 0.500, NOT_TDB17);
      $setuphold(posedge CLKB &&& re_flagB, TDB[18], 1.000, 0.500, NOT_TDB18);
      $setuphold(posedge CLKB &&& re_flagB, TDB[19], 1.000, 0.500, NOT_TDB19);
      $setuphold(posedge CLKB &&& re_flagB, TDB[20], 1.000, 0.500, NOT_TDB20);
      $setuphold(posedge CLKB &&& re_flagB, TDB[21], 1.000, 0.500, NOT_TDB21);
      $setuphold(posedge CLKB &&& re_flagB, TDB[22], 1.000, 0.500, NOT_TDB22);
      $setuphold(posedge CLKB &&& re_flagB, TDB[23], 1.000, 0.500, NOT_TDB23);
      $setuphold(posedge CLKB &&& re_flagB, TDB[24], 1.000, 0.500, NOT_TDB24);
      $setuphold(posedge CLKB &&& re_flagB, TDB[25], 1.000, 0.500, NOT_TDB25);
      $setuphold(posedge CLKB &&& re_flagB, TDB[26], 1.000, 0.500, NOT_TDB26);
      $setuphold(posedge CLKB &&& re_flagB, TDB[27], 1.000, 0.500, NOT_TDB27);
      $setuphold(posedge CLKB &&& re_flagB, TDB[28], 1.000, 0.500, NOT_TDB28);
      $setuphold(posedge CLKB &&& re_flagB, TDB[29], 1.000, 0.500, NOT_TDB29);
      $setuphold(posedge CLKB &&& re_flagB, TDB[30], 1.000, 0.500, NOT_TDB30);
      $setuphold(posedge CLKB &&& re_flagB, TDB[31], 1.000, 0.500, NOT_TDB31);
      $setuphold(posedge CLKB &&& re_flagB, AB[0], 1.000, 0.500, NOT_AB0);
      $setuphold(posedge CLKB &&& re_flagB, AB[1], 1.000, 0.500, NOT_AB1);
      $setuphold(posedge CLKB &&& re_flagB, AB[2], 1.000, 0.500, NOT_AB2);
      $setuphold(posedge CLKB &&& re_flagB, AB[3], 1.000, 0.500, NOT_AB3);
      $setuphold(posedge CLKB &&& re_flagB, AB[4], 1.000, 0.500, NOT_AB4);
      $setuphold(posedge CLKB &&& re_flagB, TAB[0], 1.000, 0.500, NOT_TAB0);
      $setuphold(posedge CLKB &&& re_flagB, TAB[1], 1.000, 0.500, NOT_TAB1);
      $setuphold(posedge CLKB &&& re_flagB, TAB[2], 1.000, 0.500, NOT_TAB2);
      $setuphold(posedge CLKB &&& re_flagB, TAB[3], 1.000, 0.500, NOT_TAB3);
      $setuphold(posedge CLKB &&& re_flagB, TAB[4], 1.000, 0.500, NOT_TAB4);
      $setuphold(posedge CLKB, TISB, 1.000, 0.500, NOT_TISB);
      $setuphold(posedge CLKB, TMSB, 1.000, 0.500, NOT_TMSB);
      $setuphold(posedge CLKB, CENB, 1.000, 0.500, NOT_CENB);
      $setuphold(posedge CLKB, TCENB, 1.000, 0.500, NOT_TCENB);
      $setuphold(posedge CLKB &&& re_flagB, DSIB, 1.000, 0.500, NOT_DSIB);
      $setuphold(posedge CLKB &&& re_flagB, SMSB, 1.000, 0.500, NOT_SMSB);

      $setuphold(posedge CLKA &&& re_flagA, AA[0], 1.000, 0.500, NOT_AA0);
      $setuphold(posedge CLKA &&& re_flagA, AA[1], 1.000, 0.500, NOT_AA1);
      $setuphold(posedge CLKA &&& re_flagA, AA[2], 1.000, 0.500, NOT_AA2);
      $setuphold(posedge CLKA &&& re_flagA, AA[3], 1.000, 0.500, NOT_AA3);
      $setuphold(posedge CLKA &&& re_flagA, AA[4], 1.000, 0.500, NOT_AA4);
      $setuphold(posedge CLKA &&& re_flagA, TAA[0], 1.000, 0.500, NOT_TAA0);
      $setuphold(posedge CLKA &&& re_flagA, TAA[1], 1.000, 0.500, NOT_TAA1);
      $setuphold(posedge CLKA &&& re_flagA, TAA[2], 1.000, 0.500, NOT_TAA2);
      $setuphold(posedge CLKA &&& re_flagA, TAA[3], 1.000, 0.500, NOT_TAA3);
      $setuphold(posedge CLKA &&& re_flagA, TAA[4], 1.000, 0.500, NOT_TAA4);
      $setuphold(posedge CLKA, TISA, 1.000, 0.500, NOT_TISA);
      $setuphold(posedge CLKA &&& re_flagA, OENA, 1.000, 0.500, NOT_OENA);
      $setuphold(posedge CLKA &&& re_flagA, TOENA, 1.000, 0.500, NOT_TOENA);
      $setuphold(posedge CLKA, TMSA, 1.000, 0.500, NOT_TMSA);
      $setuphold(posedge CLKA, CENA, 1.000, 0.500, NOT_CENA);
      $setuphold(posedge CLKA, TCENA, 1.000, 0.500, NOT_TCENA);

      $period(posedge CLKA, 3.000, NOT_CLKA_PER);
      $width(posedge CLKA, 1.000, 0, NOT_CLKA_MINH);
      $width(negedge CLKA, 1.000, 0, NOT_CLKA_MINL);

      $period(posedge CLKB, 3.000, NOT_CLKB_PER);
      $width(posedge CLKB, 1.000, 0, NOT_CLKB_MINH);
      $width(negedge CLKB, 1.000, 0, NOT_CLKB_MINL);

      $setup(posedge CLKA, posedge CLKB &&& contB_flag, 3.000, NOT_CONTB);
      $setup(posedge CLKB, posedge CLKA &&& contA_flag, 3.000, NOT_CONTA);
      $hold(posedge CLKA, posedge CLKB &&& cont_flag, 0.010, NOT_CONTB);

      (CLKA => QSQA) = (1.000, 1.000);
      (CLKA => TQA[0]) = (1.000, 1.000);
      (CLKA => TQA[1]) = (1.000, 1.000);
      (CLKA => TQA[2]) = (1.000, 1.000);
      (CLKA => TQA[3]) = (1.000, 1.000);
      (CLKA => TQA[4]) = (1.000, 1.000);
      (CLKA => TQA[5]) = (1.000, 1.000);
      (CLKA => TQA[6]) = (1.000, 1.000);
      (CLKA => TQA[7]) = (1.000, 1.000);
      (CLKA => TQA[8]) = (1.000, 1.000);
      (CLKA => TQA[9]) = (1.000, 1.000);
      (CLKA => TQA[10]) = (1.000, 1.000);
      (CLKA => TQA[11]) = (1.000, 1.000);
      (CLKA => TQA[12]) = (1.000, 1.000);
      (CLKA => TQA[13]) = (1.000, 1.000);
      (CLKA => TQA[14]) = (1.000, 1.000);
      (CLKA => TQA[15]) = (1.000, 1.000);
      (CLKA => TQA[16]) = (1.000, 1.000);
      (CLKA => TQA[17]) = (1.000, 1.000);
      (CLKA => TQA[18]) = (1.000, 1.000);
      (CLKA => TQA[19]) = (1.000, 1.000);
      (CLKA => TQA[20]) = (1.000, 1.000);
      (CLKA => TQA[21]) = (1.000, 1.000);
      (CLKA => TQA[22]) = (1.000, 1.000);
      (CLKA => TQA[23]) = (1.000, 1.000);
      (CLKA => TQA[24]) = (1.000, 1.000);
      (CLKA => TQA[25]) = (1.000, 1.000);
      (CLKA => TQA[26]) = (1.000, 1.000);
      (CLKA => TQA[27]) = (1.000, 1.000);
      (CLKA => TQA[28]) = (1.000, 1.000);
      (CLKA => TQA[29]) = (1.000, 1.000);
      (CLKA => TQA[30]) = (1.000, 1.000);
      (CLKA => TQA[31]) = (1.000, 1.000);
      (CLKA => QA[0]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[1]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[2]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[3]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[4]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[5]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[6]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[7]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[8]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[9]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[10]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[11]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[12]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[13]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[14]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[15]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[16]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[17]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[18]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[19]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[20]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[21]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[22]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[23]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[24]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[25]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[26]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[27]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[28]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[29]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[30]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (CLKA => QA[31]) = (1.000, 1.000, 0.500, 1.000, 0.500, 1.000);
      (TMSA => QA[0]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[1]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[2]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[3]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[4]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[5]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[6]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[7]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[8]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[9]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[10]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[11]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[12]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[13]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[14]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[15]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[16]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[17]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[18]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[19]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[20]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[21]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[22]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[23]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[24]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[25]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[26]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[27]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[28]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[29]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[30]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TMSA => QA[31]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[0]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[1]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[2]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[3]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[4]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[5]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[6]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[7]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[8]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[9]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[10]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[11]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[12]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[13]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[14]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[15]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[16]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[17]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[18]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[19]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[20]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[21]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[22]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[23]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[24]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[25]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[26]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[27]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[28]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[29]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[30]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TISA => QA[31]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[0]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[1]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[2]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[3]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[4]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[5]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[6]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[7]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[8]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[9]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[10]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[11]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[12]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[13]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[14]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[15]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[16]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[17]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[18]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[19]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[20]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[21]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[22]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[23]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[24]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[25]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[26]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[27]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[28]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[29]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[30]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (OENA => QA[31]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[0]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[1]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[2]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[3]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[4]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[5]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[6]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[7]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[8]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[9]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[10]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[11]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[12]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[13]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[14]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[15]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[16]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[17]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[18]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[19]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[20]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[21]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[22]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[23]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[24]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[25]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[26]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[27]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[28]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[29]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[30]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TOENA => QA[31]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[0]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[1]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[2]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[3]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[4]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[5]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[6]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[7]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[8]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[9]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[10]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[11]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[12]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[13]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[14]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[15]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[16]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[17]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[18]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[19]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[20]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[21]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[22]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[23]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[24]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[25]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[26]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[27]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[28]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[29]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[30]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (TQOENA => TQA[31]) = (1.000, 1.000, 1.000, 1.000, 1.000, 1.000);
      (CLKA => OENSQA) = (1.000, 1.000);
      (CLKA => ASQA[0]) = (1.000, 1.000);
      (CLKA => ASQA[1]) = (1.000, 1.000);
      (CLKA => ASQA[2]) = (1.000, 1.000);
      (CLKA => ASQA[3]) = (1.000, 1.000);
      (CLKA => ASQA[4]) = (1.000, 1.000);
      (CLKB => ASQB[0]) = (1.000, 1.000);
      (CLKB => ASQB[1]) = (1.000, 1.000);
      (CLKB => ASQB[2]) = (1.000, 1.000);
      (CLKB => ASQB[3]) = (1.000, 1.000);
      (CLKB => ASQB[4]) = (1.000, 1.000);
      (CLKB => CENSQB) = (1.000, 1.000);
      (CLKB => DSOB) = (1.000, 1.000);

endspecify

endmodule
`endcelldefine
